// ------------------------------------------------------------------
// File.......: chk_special.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 11:30:08 CEST 2008
// Revision...: 1.0
// Description: Checks if inputs equals special values such as 
//              infinity, nan, zero or int. Result is used for 
//              exception generation.
// ------------------------------------------------------------------

`include "defines.v"

module chk_special 
(
	fracs,	// Input from significand bus.
	exps,		// Input from exponent bus.
	format, 	// Input.
	infs,		// Output.
	ints,		// Output.
	nans,		// Output.
	zeroes	// Output.
);
   
	parameter FRACBUS = `FRACBUS;
	parameter EXPBUS = `EXPBUS;

 	// input(s)
	input [FRACBUS-1:0]	fracs;
	input [EXPBUS-1:0]	exps;
	input	[1:0]				format;

	// output(s)
	output [3:0]		infs;
	output [3:0]		ints;
	output [3:0]		nans;
	output [3:0]		zeroes;

	// wire(s)
	wire [EXPBUS/2-1:0]	exponent_a;
	wire [EXPBUS/2-1:0]	exponent_b;
	wire [FRACBUS/2-1:0]	significand_a;
	wire [FRACBUS/2-1:0]	significand_b;
	
	wire					nan_a0;
	wire					nan_a1;
	wire					nan_b0;
	wire					nan_b1;
	wire					inf_a0;
	wire					inf_a1;
	wire					inf_b0;
	wire					inf_b1;
	wire					int_a0;
	wire					int_a1;
	wire					int_b0;
	wire					int_b1;
	wire					zero_a0;
	wire					zero_a1;
	wire					zero_b0;
	wire					zero_b1;


	// reg(s)
	
	// ---------------------------------------------------------------
	// Combinational assigns.
	// ---------------------------------------------------------------
	
	// fracs[1*(`FP16SW+1)-2:0*(`FP16SW+1) because significands are
	// now extended to 11, 24 and 53 bits included the implict bit.
	
	// Assign invalid inputs.
	assign nan_a0 = 
		(format == `FP16) ? 
			(&exps[1*(`FP16EW)-1:0*(`FP16EW)]) & 
			(|fracs[1*(`FP16SW+1)-2:0*(`FP16SW+1)]) :
		(format == `FP32) ? 
			(&exps[1*(`FP32EW)-1:0*(`FP32EW)]) & 
			(|fracs[1*(`FP32SW+1)-2:0*(`FP32SW+1)]) : 
		(format == `FP64) ? 
			(&exps[1*(`FP64EW)-1:0*(`FP64EW)]) & 
			(|fracs[1*(`FP64SW+1)-2:0*(`FP64SW+1)]) : 1'b0;
	
	assign nan_b0 =
		(format == `FP16) ?
			(&exps[2*(`FP16EW)-1:1*(`FP16EW)]) &
			(|fracs[2*(`FP16SW+1)-2:1*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[2*(`FP32EW)-1:1*(`FP32EW)]) &
			(|fracs[2*(`FP32SW+1)-2:1*(`FP32SW+1)]) :
		(format == `FP64) ?
			(&exps[2*(`FP64EW)-1:1*(`FP64EW)]) &
			(|fracs[2*(`FP64SW+1)-2:1*(`FP64SW+1)]) : 1'b0;
	
	assign nan_a1 =
		(format == `FP16) ?
			(&exps[3*(`FP16EW)-1:2*(`FP16EW)]) &
			(|fracs[3*(`FP16SW+1)-2:2*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[3*(`FP32EW)-1:2*(`FP32EW)]) &
			(|fracs[3*(`FP32SW+1)-2:2*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	assign nan_b1 =
		(format == `FP16) ?
			(&exps[4*(`FP16EW)-1:3*(`FP16EW)]) &
			(|fracs[4*(`FP16SW+1)-2:3*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[4*(`FP32EW)-1:3*(`FP32EW)]) &
			(|fracs[4*(`FP32SW+1)-2:3*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	
	// Assign infinity inputs.
	assign inf_a0 =
		(format == `FP16) ?
			(&exps[1*(`FP16EW)-1:0*(`FP16EW)]) &
			(~|fracs[1*(`FP16SW+1)-2:0*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[1*(`FP32EW)-1:0*(`FP32EW)]) &
			(~|fracs[1*(`FP32SW+1)-2:0*(`FP32SW+1)]) :
		(format == `FP64) ?
			(&exps[1*(`FP64EW)-1:0*(`FP64EW)]) &
			(~|fracs[1*(`FP64SW+1)-2:0*(`FP64SW+1)]) : 1'b0;
	
	assign inf_b0 =
		(format == `FP16) ?
			(&exps[2*(`FP16EW)-1:1*(`FP16EW)]) &
			(~|fracs[2*(`FP16SW+1)-2:1*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[2*(`FP32EW)-1:1*(`FP32EW)]) &
			(~|fracs[2*(`FP32SW+1)-2:1*(`FP32SW+1)]) :
		(format == `FP64) ?
			(&exps[2*(`FP64EW)-1:1*(`FP64EW)]) &
			(~|fracs[2*(`FP64SW+1)-2:1*(`FP64SW+1)]) : 1'b0;
	
	assign inf_a1 =
		(format == `FP16) ?
			(&exps[3*(`FP16EW)-1:2*(`FP16EW)]) &
			(~|fracs[3*(`FP16SW+1)-2:2*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[3*(`FP32EW)-1:2*(`FP32EW)]) &
			(~|fracs[3*(`FP32SW+1)-2:2*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	assign inf_b1 =
		(format == `FP16) ?
			(&exps[4*(`FP16EW)-1:3*(`FP16EW)]) &
			(~|fracs[4*(`FP16SW+1)-2:3*(`FP16SW+1)]) :
		(format == `FP32) ?
			(&exps[4*(`FP32EW)-1:3*(`FP32EW)]) &
			(~|fracs[4*(`FP32SW+1)-2:3*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	
	// Assign zero inputs.
	assign zero_a0 =
		(format == `FP16) ?
			(~|exps[1*(`FP16EW)-1:0*(`FP16EW)]) &
			(~|fracs[1*(`FP16SW+1)-2:0*(`FP16SW+1)]) :
		(format == `FP32) ?
			(~|exps[1*(`FP32EW)-1:0*(`FP32EW)]) &
			(~|fracs[1*(`FP32SW+1)-2:0*(`FP32SW+1)]) :
		(format == `FP64) ?
			(~|exps[1*(`FP64EW)-1:0*(`FP64EW)]) &
			(~|fracs[1*(`FP64SW+1)-2:0*(`FP64SW+1)]) : 1'b0;
	
	assign zero_b0 =
		(format == `FP16) ?
			(~|exps[2*(`FP16EW)-1:1*(`FP16EW)]) &
			(~|fracs[2*(`FP16SW+1)-2:1*(`FP16SW+1)]) :
		(format == `FP32) ?
			(~|exps[2*(`FP32EW)-1:1*(`FP32EW)]) &
			(~|fracs[2*(`FP32SW+1)-2:1*(`FP32SW+1)]) :
		(format == `FP64) ?
			(~|exps[2*(`FP64EW)-1:1*(`FP64EW)]) &
			(~|fracs[2*(`FP64SW+1)-2:1*(`FP64SW+1)]) : 1'b0;
	
	assign zero_a1 =
		(format == `FP16) ?
			(~|exps[3*(`FP16EW)-1:2*(`FP16EW)]) &
			(~|fracs[3*(`FP16SW+1)-2:2*(`FP16SW+1)]) :
		(format == `FP32) ?
			(~|exps[3*(`FP32EW)-1:2*(`FP32EW)]) &
			(~|fracs[3*(`FP32SW+1)-2:2*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	assign zero_b1 =
		(format == `FP16) ?
			(~|exps[4*(`FP16EW)-1:3*(`FP16EW)]) &
			(~|fracs[4*(`FP16SW+1)-2:3*(`FP16SW+1)]) :
		(format == `FP32) ?
			(~|exps[4*(`FP32EW)-1:3*(`FP32EW)]) &
			(~|fracs[4*(`FP32SW+1)-2:3*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	
	// Assign integer inputs.
	assign int_a0 =
		(format == `FP16) ?
			(|exps[1*(`FP16EW)-1:0*(`FP16EW)]) &
			(~|fracs[1*(`FP16SW+1)-2:0*(`FP16SW+1)]) :
		(format == `FP32) ?
			(|exps[1*(`FP32EW)-1:0*(`FP32EW)]) &
			(~|fracs[1*(`FP32SW+1)-2:0*(`FP32SW+1)]) :
		(format == `FP64) ?
			(|exps[1*(`FP64EW)-1:0*(`FP64EW)]) &
			(~|fracs[1*(`FP64SW+1)-2:0*(`FP64SW+1)]) : 1'b0;
	
	assign int_b0 =
		(format == `FP16) ?
			(|exps[2*(`FP16EW)-1:1*(`FP16EW)]) &
			(~|fracs[2*(`FP16SW+1)-2:1*(`FP16SW+1)]) :
		(format == `FP32) ?
			(|exps[2*(`FP32EW)-1:1*(`FP32EW)]) &
			(~|fracs[2*(`FP32SW+1)-2:1*(`FP32SW+1)]) :
		(format == `FP64) ?  
			(|exps[2*(`FP64EW)-1:1*(`FP64EW)]) &
			(~|fracs[2*(`FP64SW+1)-2:1*(`FP64SW+1)]) : 1'b0;
	
	assign int_a1 =
		(format == `FP16) ?
			(|exps[3*(`FP16EW)-1:2*(`FP16EW)]) &
			(~|fracs[3*(`FP16SW+1)-2:2*(`FP16SW+1)]) :
		(format == `FP32) ?
			(|exps[3*(`FP32EW)-1:2*(`FP32EW)]) &
			(~|fracs[3*(`FP32SW+1)-2:2*(`FP32SW+1)]) :
		(format == `FP64) ? 1'b0 : 1'b0;
	
	assign int_b1 =
		(format == `FP16) ?
			(|exps[4*(`FP16EW)-1:3*(`FP16EW)]) &
			(~|fracs[4*(`FP16SW+1)-2:3*(`FP16SW+1)]) :
		(format == `FP32) ?
			(|exps[4*(`FP32EW)-1:3*(`FP32EW)]) &
		(~|fracs[4*(`FP32SW+1)-2:3*(`FP32SW+1)]) :
			(format == `FP64) ? 1'b0 : 1'b0;
	
		
	// Assign outputs.
	assign infs[0] = inf_a0;
	assign infs[1] = inf_b0;
	assign infs[2] = inf_a1;
	assign infs[3] = inf_b1;
	assign ints[0] = int_a0;
	assign ints[1] = int_b0;
	assign ints[2] = int_a1;
	assign ints[3] = int_b1;
	assign nans[0] = nan_a0;
	assign nans[1] = nan_b0;
	assign nans[2] = nan_a1;
	assign nans[3] = nan_b1;
	assign zeroes[0] = zero_a0;
	assign zeroes[1] = zero_b0;
	assign zeroes[2] = zero_a1;
	assign zeroes[3] = zero_b1;
	
endmodule // chk_special
