// ------------------------------------------------------------------
// File.......: rne.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 11:10:54 CEST 2008
// Revision...: 1.0
// Description: Rounding and exception unit. Rounds, normalizes and 
//              postnormlizes the result from the computation, and 
//              generates exceptions if needed.
// ------------------------------------------------------------------

`include "defines.v"

module rne 
(
	frac,			// Input. Fractional part from significand multiplication.
	sign,			// Input. Sign from sign computation.
	exp,			// Input. Biased exponent from exponent addition.
	specials,	// Input. NaNs, infinities, zeros..
	mode,			// Input. Rounding mode.
	result,		// Output. Rounded result or special value.
	exceps		// Output. Exceptions.
);
   	
	parameter SW = 52;
	parameter EW = 11;

 	// input(s)
	input [2*SW+1:0]	frac;
	input [EW+1:0]		exp;
	input 				sign;
	input [7:0]			specials;
	input [1:0]			mode;

	// output(s)
	output [SW+EW:0]	result;
	output [3:0]		exceps;

	// wire(s)
	wire 					normalize;
	wire 					postnormalize;
	wire 					lsb;
	wire 					round;
	wire 					sticky;
	wire 					roundup;
	wire 					rounded;
	wire 					ovf_ab;
	wire 					ovf_biased;
	wire 					ovf_postnorm;
	wire 					round_to_nearest_even;
	wire 					round_to_infinity;
	wire 					round_to_zero;
	wire 					nan_a;
	wire 					nan_b;
	wire 					int_a;
	wire 					int_b;
	wire 					inf_a;
	wire 					inf_b;
	wire 					zero_a;
	wire 					zero_b;
	wire					int_times_inf;
	wire 					invalid;
	wire					overflow;
	wire					overflow_tmp;
	wire					underflow;
	wire					underflow_tmp;
	wire					inexact;
	wire					exp_zero;
	wire [SW:0]			significand;
	wire [SW:0]			significand_tmp;
	wire [SW:0]			significand_plus_ulp;
	wire [EW:0] 		exponent;
	wire [SW+EW:0] 	result_tmp;
	wire [SW+EW:0] 	product_nan;
	wire [SW+EW:0] 	product_zero;
	wire [SW+EW:0] 	product_large;
	wire [SW+EW:0] 	product_overflow;
	
	// reg(s)	
		
	// Round and normalize / Postnormalize.
	// ---------------------------------------------------------------
	
	// Normalize if result from multiplier lies in [2,4)
	assign normalize = frac[2*SW+1];
	
	assign significand_tmp = 
		normalize ? 
		frac[2*SW:SW] >> 1: frac[2*SW:SW];
	
	assign exponent = 
		normalize ? 
		exp[EW-1:0] + 1 : exp[EW-1:0];
	
	
	// Assign rounding bits.
	assign lsb = 
		normalize ? 
		frac[SW+1] : 
		frac[SW];
	
	assign round = 
		normalize ? 
		frac[SW] : 
		frac[SW-1];
	
	assign sticky = 
		normalize ? 
		|frac[SW-1:0] : 
		|frac[SW-2:0];
		
	// Reduce to three rounding modes.
	assign round_to_nearest_even = 
		(round & (lsb | sticky)) & !(|mode);

	assign round_to_infinity = 
		(!sign&(!mode[1]&mode[0])|sign&(mode[1]&!mode[0])) & 
		(round|sticky);

	assign round_to_zero = 
		(sign&(~mode[1]&mode[0])|~sign&(mode[1]&~mode[0]))|&mode;
	
	// Round-up if necessary.
	assign significand_plus_ulp = significand_tmp + 1'b1;
	assign roundup = round_to_infinity | round_to_nearest_even;
	assign significand = 
		roundup ? 
			significand_plus_ulp : significand_tmp;
								
	// Post-normalize if result after rounding lies in [2,4).
	assign postnormalize = !significand[SW] & significand_tmp[SW];
	assign result_tmp = 
		postnormalize ? 
		{sign, exponent[EW-1:0] + 1'b1, significand[SW:1]} : 
		{sign, exponent[EW-1:0], significand[SW-1:0]};
	
	// Inexact if result was rounded.
	assign rounded = round | sticky;
	
	assign ovf_postnorm = 
		exponent[EW] | &exponent[EW-1:0]&(normalize|postnormalize);
	
	
	// Generate exceptions.
	// ---------------------------------------------------------------
	
	assign ovf_ab = exp[EW+1];
	assign ovf_biased = exp[EW];
	
	// Invalid inputs from chk_special.
	assign nan_a = specials[0];
	assign nan_b = specials[1];
	assign inf_a = specials[2];
	assign inf_b = specials[3];
	assign zero_a = specials[4];
	assign zero_b = specials[5];
	assign int_a = specials[6];
	assign int_b = specials[7];
	
	
	// Generate exceptions.
	assign int_times_inf = (int_a&inf_b)|(int_b&inf_a); 
	
	assign invalid = 
		(nan_a | nan_b)|
		(zero_a&inf_b | zero_b&inf_a)| 
		(inf_a | inf_b)&!int_times_inf;
	
	assign inexact = 
		(rounded & (!invalid)|
		overflow_tmp| 
		round_to_zero&overflow_tmp| 
		underflow&(!(zero_a|zero_b)))&!int_times_inf;

	assign underflow = 
		(~ovf_ab&ovf_biased)| 
		(~|result_tmp[SW+EW-1:SW])&!(ovf_ab&ovf_biased|ovf_postnorm) &
		!overflow&!invalid|(zero_a|zero_b)&!(nan_a|nan_b|inf_a|inf_b);
		
	// If overflow occurs and rounding mode equals round-to zero, 
	// result shall be rounded to largest representative number. 
	// e.x 0111101111111111.
	
	assign overflow_tmp = 
		((ovf_ab&ovf_biased|ovf_postnorm&!underflow)|
		&result_tmp[SW+EW-1:SW]&!underflow)&!invalid;
		
	assign overflow = overflow_tmp&!round_to_zero | int_times_inf;
		
	
	// Compute special results.
	assign product_nan = 
		{1'b0, {EW{1'b1}}, {(SW-1){1'b0}}, 1'b1};
		
	assign product_zero = 
		{result_tmp[SW+EW], {(SW+EW){1'b0}}};
				
	assign product_overflow = 
		{result_tmp[SW+EW], {EW{1'b1}}, {(SW){1'b0}}};

	assign product_large = 
		{result_tmp[SW+EW], {(EW-1){1'b1}}, 1'b0, {(SW){1'b1}}};
	
	// Final product decided by exceptions.
	assign result = 
		invalid ? product_nan : 
		overflow ? product_overflow : 
		underflow ? product_zero : 
		round_to_zero & overflow_tmp & !int_times_inf ? product_large : 
		result_tmp;
	
	assign exceps[0] = invalid;
	assign exceps[1] = inexact;
	assign exceps[2] = overflow;
	assign exceps[3] = underflow;
	
endmodule // rne
