// ------------------------------------------------------------------
// File.......: rne_unit.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 12:19:50 CEST 2008
// Revision...: 1.0
// Description: Rounding, normalizing and exception unit.
// ------------------------------------------------------------------

`include "defines.v"

module rne_unit 
(
	fracs,	// Input from fraction bus.
	exps,		// Input form exponent bus.
	signs,	// Input from sign bus.
	format,	// Input from instrucion register.
	special, // Input form check special.
	mode,		// Input from mode register.
	exceps,	// Output exceptions.
	result	// Output. Rounded result.
);
   
 	// input(s)
	input [`FRACBUS-1:0]			fracs;
	input [`EXPBUS/2+3:0]		exps;
	input [`SIGNBUS/2-1:0]		signs;
	input [1:0]						format;
	input [1:0]						mode;
	input [15:0]					special;

	// output(s)
	output [`BUS/2-1:0]			result;
	output [7:0]					exceps;

	// wire(s)
	wire [2*`FP64SW+1:0]			frac_rne_0;
	wire 								sign_rne_0;
	wire [`FP64EW+1:0]			exp_rne_0;
	wire [7:0]						specials_rne_0;
	wire [`FP64SW+`FP64EW:0]	result_rne_0;
	wire [3:0]						exceps_rne_0;
	wire [2*`FP64SW+1:0]			frac_rne_1;
	wire 								sign_rne_1;
	wire [`FP64EW+1:0]			exp_rne_1;
	wire [7:0]						specials_rne_1;
	wire [`FP64SW+`FP64EW:0]	result_rne_1;
	wire [3:0]						exceps_rne_1;
	wire [1:0]						fp16_overflow;
	wire [1:0]						fp16_underflow;
	wire [1:0]						fp16_inexact;
	wire [1:0]						fp16_invalid;
	wire [1:0]						fp32_overflow;
	wire [1:0]						fp32_underflow;
	wire [1:0]						fp32_inexact;
	wire [1:0]						fp32_invalid;
	wire [1:0]						fp64_overflow;
	wire [1:0]						fp64_underflow;
	wire [1:0]						fp64_inexact;
	wire [1:0]						fp64_invalid;
	wire [3:0]						exceps_fp16_rne_0;
	wire [3:0]						exceps_fp16_rne_1;
	wire [3:0]						exceps_fp32_rne_0;
	wire [3:0]						exceps_fp32_rne_1;
	wire [3:0]						exceps_fp64_rne_0;
	wire [7:0]						fp16_exceps;
	wire [7:0]						fp32_exceps;
	wire [7:0]						fp64_exceps;
	wire [2*`FP16SW+1:0]			frac_fp16_rne_0;
	wire [2*`FP16SW+1:0]			frac_fp16_rne_1;
	wire [2*`FP32SW+1:0]			frac_fp32_rne_0;
	wire [2*`FP32SW+1:0]			frac_fp32_rne_1;
	wire [2*`FP64SW+1:0]			frac_fp64_rne_0;
	wire [`FP16EW+1:0]			exp_fp16_rne_0;
	wire [`FP16EW+1:0]			exp_fp16_rne_1;
	wire [`FP32EW+1:0]			exp_fp32_rne_0;
	wire [`FP32EW+1:0]			exp_fp32_rne_1;
	wire [`FP64EW+1:0]			exp_fp64_rne_0;
	wire [`FP16W-1:0]				result_fp16_rne_0;
	wire [`FP16W-1:0]				result_fp16_rne_1;
	wire [`FP32W-1:0]				result_fp32_rne_0;
	wire [`FP32W-1:0]				result_fp32_rne_1;
	wire [`FP64W-1:0]				result_fp64_rne_0;
	wire [2*`FP16W-1:0]			result_fp16_rne;
	wire [2*`FP32W-1:0]			result_fp32_rne;
	wire [2*`FP64W-1:0]			result_fp64_rne;
	
	// reg(s)
	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------

	rne #(`FP16SW, `FP16EW) fp16_rne_0
	(
		.frac			(frac_fp16_rne_0),
		.sign			(sign_rne_0),
		.exp			(exp_fp16_rne_0),
		.specials	(specials_rne_0),
		.mode			(mode),
		.result		(result_fp16_rne_0),
		.exceps		(exceps_fp16_rne_0)
	);
	
	rne #(`FP16SW, `FP16EW) fp16_rne_1
	(
		.frac			(frac_fp16_rne_1),
		.sign			(sign_rne_1),
		.exp			(exp_fp16_rne_1),
		.specials	(specials_rne_1),
		.mode			(mode),
		.result		(result_fp16_rne_1),
		.exceps		(exceps_fp16_rne_1)
	);
	
	rne #(`FP32SW, `FP32EW) fp32_rne_0
	(
		.frac			(frac_fp32_rne_0),
		.sign			(sign_rne_0),
		.exp			(exp_fp32_rne_0),
		.specials	(specials_rne_0),
		.mode			(mode),
		.result		(result_fp32_rne_0),
		.exceps		(exceps_fp32_rne_0)
	);
	
	rne #(`FP32SW, `FP32EW) fp32_rne_1
	(
		.frac			(frac_fp32_rne_1),
		.sign			(sign_rne_1),
		.exp			(exp_fp32_rne_1),
		.specials	(specials_rne_1),
		.mode			(mode),
		.result		(result_fp32_rne_1),
		.exceps		(exceps_fp32_rne_1)
	);
	
	rne #(`FP64SW, `FP64EW) fp64_rne_0
	(
		.frac			(frac_fp64_rne_0),
		.sign			(sign_rne_0),
		.exp			(exp_fp64_rne_0),
		.specials	(specials_rne_0),
		.mode			(mode),
		.result		(result_fp64_rne_0),
		.exceps		(exceps_fp64_rne_0)
	);
	
	
	// ---------------------------------------------------------------
	// Combinalional assign.
	// ---------------------------------------------------------------
	
	// Inputs to rounding logic.
	assign frac_fp16_rne_0 = fracs[2*(`FP16SW+1)-1:0*(`FP16SW+1)];
	assign frac_fp16_rne_1 = fracs[4*(`FP16SW+1)-1:2*(`FP16SW+1)];
	assign frac_fp32_rne_0 = fracs[2*(`FP32SW+1)-1:0*(`FP32SW+1)];
	assign frac_fp32_rne_1 = fracs[4*(`FP32SW+1)-1:2*(`FP32SW+1)];
	assign frac_fp64_rne_0 = fracs[2*(`FP64SW+1)-1:0*(`FP64SW+1)];
	
	// Two msb bits represents the overflow bits during exponent 
	// addition.
	assign exp_fp16_rne_0 = exps[1*(`FP16EW+1):0*(`FP16EW+2)];
	assign exp_fp16_rne_1 = exps[2*(`FP16EW+1)+1:1*(`FP16EW+2)];
	assign exp_fp32_rne_0 = exps[1*(`FP32EW+1):0*(`FP32EW+2)];
	assign exp_fp32_rne_1 = exps[2*(`FP32EW+1)+1:1*(`FP32EW+2)];
	assign exp_fp64_rne_0 = exps[1*(`FP64EW+1):0*(`FP64EW+2)];
	
	assign sign_rne_0 = signs[0];
	assign sign_rne_1 = signs[1];
	
	assign specials_rne_0 = 
		{special[13], special[12], special[9], special[8],
		 special[5], special[4], special[1], special[0]};

	assign specials_rne_1 = 
		{special[15], special[14], special[11], special[10],
		 special[7], special[6], special[3], special[2]};
	
	
	// Outputs from rounding logic.
	assign result_fp16_rne = {result_fp16_rne_1, result_fp16_rne_0};
	assign result_fp32_rne = {result_fp32_rne_1, result_fp32_rne_0};
	assign result_fp64_rne = result_fp64_rne_0;
							
	assign fp16_underflow = 
		{exceps_fp16_rne_1[3], exceps_fp16_rne_0[3]};
	assign fp16_overflow = 
		{exceps_fp16_rne_1[2], exceps_fp16_rne_0[2]};
	assign fp16_inexact = 
		{exceps_fp16_rne_1[1], exceps_fp16_rne_0[1]};
	assign fp16_invalid = 
		{exceps_fp16_rne_1[0], exceps_fp16_rne_0[0]};
	
	assign fp32_underflow = 
		{exceps_fp32_rne_1[3], exceps_fp32_rne_0[3]};
	assign fp32_overflow = 
		{exceps_fp32_rne_1[2], exceps_fp32_rne_0[2]};
	assign fp32_inexact = 
		{exceps_fp32_rne_1[1], exceps_fp32_rne_0[1]};
	assign fp32_invalid = 
		{exceps_fp32_rne_1[0], exceps_fp32_rne_0[0]};
	
	assign fp64_underflow = 
		{1'b0, exceps_fp64_rne_0[3]};
	assign fp64_overflow = 
		{1'b0, exceps_fp64_rne_0[2]};
	assign fp64_inexact = 
		{1'b0, exceps_fp64_rne_0[1]};
	assign fp64_invalid = 
		{1'b0, exceps_fp64_rne_0[0]};
	
	assign fp16_exceps = 
		{fp16_underflow, fp16_overflow, fp16_inexact, fp16_invalid};
	assign fp32_exceps = 
		{fp32_underflow, fp32_overflow, fp32_inexact, fp32_invalid};
	assign fp64_exceps = 
		{fp64_underflow, fp64_overflow, fp64_inexact, fp64_invalid};
	
	assign exceps = 
		(format == `FP16) ? fp16_exceps :
		(format == `FP32) ? fp32_exceps :
		(format == `FP64) ? fp64_exceps : 0;
						
	assign result = 
		(format == `FP16) ? result_fp16_rne :
		(format == `FP32) ? result_fp32_rne :
		(format == `FP64) ? result_fp64_rne : 0;

endmodule // rne_unit
