// ------------------------------------------------------------------
// File.......: sel_input.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 10:54:28 CEST 2008
// Revision...: 1.0
// Description: Selects data from the input registers and puts it on 
//              the exponent, significand and sign busses.
// ------------------------------------------------------------------

`include "defines.v"

module sel_input 
(
	drh,		// Input from data-register high (DRH0).
	drl,		// Input from data-register low (DRL0).
	format,	// Input form instrucion(format) register.
	signs,	// Output to sign bus.
	exps,		// Output to exponent bus.
	fracs		// Output to significand bus.
);
   
	parameter WIDTH 		= `BUS/2;
	parameter SIGNBUS 	= `SIGNBUS;
	parameter EXPBUS 		= `EXPBUS;
	parameter FRACBUS 	= `FRACBUS;

 	// input(s)
	input [WIDTH-1:0]		drh;
	input [WIDTH-1:0]		drl;
	input [1:0]				format;

	// output(s)
	output [SIGNBUS-1:0]	signs;
	output [EXPBUS-1:0]	exps;
	output [FRACBUS-1:0]	fracs;

	// wire(s)

	// reg(s)
	reg [SIGNBUS-1:0]		signs_tmp;
	reg [EXPBUS-1:0]		exps_tmp;
	reg [FRACBUS-1:0]		fracs_tmp;
	

	// ---------------------------------------------------------------
	// Combinational logic.
	// ---------------------------------------------------------------
	
	always @ (drh or drl or format) begin
		signs_tmp = 0;
		exps_tmp = 0;
		fracs_tmp = 0;
		
		case (format)
			`FP16: begin
				signs_tmp = 
					{drl[4*`FP16W-1], drl[3*`FP16W-1], 
					 drl[2*`FP16W-1], drl[1*`FP16W-1]};
								
				exps_tmp = 
					{drl[4*(`FP16W)-2:3*`FP16W+`FP16SW], 
					 drl[3*(`FP16W)-2:2*`FP16W+`FP16SW], 
					 drl[2*(`FP16W)-2:1*`FP16W+`FP16SW], 
					 drl[1*(`FP16W)-2:0*`FP16W+`FP16SW]};
								
				fracs_tmp = 
					{1'b1, drl[4*`FP16W-`FP16EW-2:3*`FP16W], 
					 1'b1, drl[3*`FP16W-`FP16EW-2:2*`FP16W],
					 1'b1, drl[2*`FP16W-`FP16EW-2:1*`FP16W], 
					 1'b1, drl[1*`FP16W-`FP16EW-2:0*`FP16W]};
			end
			`FP32: begin
				signs_tmp = 
					{drh[2*`FP32W-1], drh[1*`FP32W-1], 
					 drl[2*`FP32W-1], drl[1*`FP32W-1]};
								
				exps_tmp = 
					{drh[2*(`FP32W)-2:1*`FP32W+`FP32SW], 
					 drh[1*(`FP32W)-2:0*`FP32W+`FP32SW], 
					 drl[2*(`FP32W)-2:1*`FP32W+`FP32SW], 
					 drl[1*(`FP32W)-2:0*`FP32W+`FP32SW]};
								
				fracs_tmp = 
					{1'b1, drh[2*`FP32W-`FP32EW-2:1*`FP32W], 
					 1'b1, drh[1*`FP32W-`FP32EW-2:0*`FP32W],
					 1'b1, drl[2*`FP32W-`FP32EW-2:1*`FP32W], 
					 1'b1, drl[1*`FP32W-`FP32EW-2:0*`FP32W]};
			end
			`FP64: begin
				signs_tmp = 
					{1'b0, 1'b0, drh[1*`FP64W-1], drl[1*`FP64W-1]};
								
				exps_tmp = 
					{drh[1*(`FP64W)-2:0*`FP64W+`FP64SW], 
					 drl[1*(`FP64W)-2:0*`FP64W+`FP64SW]};
								
				fracs_tmp = 
					{1'b1, drh[1*`FP64W-`FP64EW-2:0*`FP64W], 
					 1'b1, drl[1*`FP64W-`FP64EW-2:0*`FP64W]};
			end
		endcase
	end
		

	assign signs = signs_tmp;
	assign exps = exps_tmp;
	assign fracs = fracs_tmp;
	
endmodule // sel_input
