// ------------------------------------------------------------------
// File.......: rne64.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 11:10:54 CEST 2008
// Revision...: 1.0
// Description: Rounding and exception unit. Rounds, normalizes and 
//              postnormlizes the result from the computation, and 
//              generates exceptions if needed.
// ------------------------------------------------------------------

`include "defines.v"

module rne64 
(
	frac,			// Input. Fractional part from multiplication.
	sign,			// Input. Sign from sign computation.
	exp,			// Input. Biased exponent from exponent addition.
	specials,	// Input. NaNs, infinities, zeros..
	format,		// Input.
	mode,			// Input. Rounding mode.
	result,		// Output. Rounded result or special value.
	exceps		// Output. Exceptions.
);
   	
	parameter SW = 52;
	parameter EW = 11;
	
 	// input(s)
	input [2*SW+1:0]	frac;
	input [EW+1:0]		exp;
	input 				sign;
	input [7:0]			specials;
	input [1:0]			mode;
	input [1:0]			format;

	// output(s)
	output [SW+EW:0]	result;
	output [3:0]		exceps;

	// wire(s)
	wire 					normalize;
	wire 					postnormalize;
	wire 					lsb;
	wire 					round;
	wire 					sticky;
	wire 					roundup;
	wire 					rounded;
	wire 					ovf_ab;
	wire 					ovf_biased;
	wire 					ovf_postnorm;
	wire 					round_to_nearest_even;
	wire 					round_to_infinity;
	wire 					round_to_zero;
	wire 					nan_a;
	wire 					nan_b;
	wire 					int_a;
	wire 					int_b;
	wire 					inf_a;
	wire 					inf_b;
	wire 					zero_a;
	wire 					zero_b;
	wire					int_times_inf;
	wire 					invalid;
	wire					overflow;
	wire					overflow_tmp;
	wire					underflow;
	wire					underflow_tmp;
	wire					inexact;
	wire					exp_zero;
	wire [SW:0]			significand;
	wire [SW:0]			significand_tmp;
	wire [SW:0]			significand_plus_ulp;
	wire [EW:0] 		exponent;
	wire [EW:0] 		exponent_tmp;
	wire [SW+EW:0] 	result_tmp;
	wire [SW+EW:0] 	product_nan;
	wire [SW+EW:0] 	product_zero;
	wire [SW+EW:0] 	product_large;
	wire [SW+EW:0] 	product_overflow;
	wire [SW+EW:0] 	product_min;
	
	// reg(s)
	
		
	// Round and normalize / Postnormalize.
	// ---------------------------------------------------------------
	
	// Normalize if result from multiplier lies in [2,4)
	assign normalize = frac[2*SW+1];
	
	assign significand_tmp = 
		normalize ? 
		frac[2*SW:SW] >> 1: frac[2*SW:SW];
	
	assign exponent_tmp = 
		(format == `FP16) ? 
		normalize ?
		exp[`FP16EW-1:0] + 1 : exp[`FP16EW-1:0] :
		(format == `FP32) ? 
		normalize ?
		exp[`FP32EW-1:0] + 1 : exp[`FP32EW-1:0] :
		(format == `FP64) ? 
		normalize ?
		exp[`FP64EW-1:0] + 1 : exp[`FP64EW-1:0] : 0; 

	
	// Assign rounding bits.
	assign lsb = 
		(format == `FP16) ? 
		normalize ? 
		frac[95] : 
		frac[94] : 
		(format == `FP32) ? 
		normalize ? 
		frac[82] : 
		frac[81] :
		(format == `FP64) ? 
		normalize ? 
		frac[53] : 
		frac[52] : 0;
		
		
	assign round = 
		(format == `FP16) ? 
		normalize ? 
		frac[94] : 
		frac[93] : 
		(format == `FP32) ? 
		normalize ? 
		frac[81] : 
		frac[80] :
		(format == `FP64) ? 
		normalize ? 
		frac[52] : 
		frac[51] : 0;
	
	assign sticky = 
		(format == `FP16) ? 
		normalize ? 
		|frac[93:84] : 
		|frac[92:83] : 
		(format == `FP32) ? 
		normalize ? 
		|frac[80:59] : 
		|frac[79:58] :
		(format == `FP64) ? 
		normalize ? 
		|frac[51:1] : 
		|frac[50:0] : 0;
		
	// Reduce to three rounding modes.
	assign round_to_nearest_even = 
		(round & (lsb | sticky)) & !(|mode);

	assign round_to_infinity = 
		(!sign&(!mode[1]&mode[0])|sign&(mode[1]&!mode[0])) & 
		(round|sticky);

	assign round_to_zero = 
		(sign&(~mode[1]&mode[0])|~sign&(mode[1]&~mode[0]))|&mode;
	
	// Round-up if necessary.
	assign significand_plus_ulp = 
		(format == `FP16) ? 
		significand_tmp[SW:SW-`FP16SW] + 1'b1 :
		(format == `FP32) ? 
		significand_tmp[SW:SW-`FP32SW] + 1'b1 :
		(format == `FP64) ? 
		significand_tmp[SW:SW-`FP64SW] + 1'b1 : 0;
		
	assign roundup = round_to_infinity | round_to_nearest_even;
	assign significand = 
		(format == `FP16) ?
		roundup ? 
		significand_plus_ulp : significand_tmp[SW:SW-`FP16SW] :
		(format == `FP32) ?
		roundup ? 
		significand_plus_ulp : significand_tmp[SW:SW-`FP32SW] :
		(format == `FP64) ?
		roundup ? 
		significand_plus_ulp : significand_tmp[SW:SW-`FP64SW] : 0;
								
	// Post-normalize if result after rounding lies in [2,4).
	assign postnormalize = 
		(format == `FP16) ?  
		!significand[`FP16SW]&significand_tmp[SW] : 
		(format == `FP32) ?  
		!significand[`FP32SW]&significand_tmp[SW] : 
		(format == `FP64) ?  
		!significand[`FP64SW]&significand_tmp[SW] : 0;
		
	assign exponent = 
		postnormalize ?
		exponent_tmp + 1 :
		exponent_tmp;
		
		
	assign result_tmp =
		(format == `FP16) ?  
		postnormalize ? 
			{sign, exponent[`FP16EW-1:0], significand[`FP16SW-1:0]} : 
			{sign, exponent[`FP16EW-1:0], significand[`FP16SW-1:0]} :
		(format == `FP32) ?  
		postnormalize ?
			{sign, exponent[`FP32EW-1:0], significand[`FP32SW-1:0]} : 
			{sign, exponent[`FP32EW-1:0], significand[`FP32SW-1:0]} :
		(format == `FP64) ? 
		postnormalize ?
			{sign, exponent[`FP64EW-1:0], significand[`FP64SW-1:0]} : 
			{sign, exponent[`FP64EW-1:0], significand[`FP64SW-1:0]} : 
		0;
		
	// Inexact if result was rounded.
	assign rounded = round | sticky;
	
	assign ovf_postnorm = 
		(format == `FP16) ? 
			exponent[`FP16EW] | 
			&exponent[`FP16EW-1:0]&(normalize|postnormalize) :
		(format == `FP32) ? 
			exponent[`FP32EW] | 
			&exponent[`FP32EW-1:0]&(normalize|postnormalize) :
		(format == `FP64) ? 
			exponent[`FP64EW] |
			&exponent[`FP64EW-1:0]&(normalize|postnormalize) :
		0;
	
	
	// Generate exceptions.
	// -----------------------------------------------------------------------
	
	assign ovf_ab = 
		(format == `FP16) ?
			exp[`FP16EW+1] :
		(format == `FP32) ?
			exp[`FP32EW+1] : 
		(format == `FP64) ?
			exp[`FP64EW+1] : 0;
		
	assign ovf_biased = 
		(format == `FP16) ?
			exp[`FP16EW] :
		(format == `FP32) ?
			exp[`FP32EW] : 
		(format == `FP64) ?
			exp[`FP64EW] : 0;
	
	// Invalid inputs from chk_special.
	assign nan_a = specials[0];
	assign nan_b = specials[1];
	assign inf_a = specials[2];
	assign inf_b = specials[3];
	assign zero_a = specials[4];
	assign zero_b = specials[5];
	assign int_a = specials[6];
	assign int_b = specials[7];
	
	
	// Generate exceptions.
	assign int_times_inf = (int_a&inf_b)|(int_b&inf_a); 
	
	assign invalid = 
		(nan_a | nan_b)|
		(zero_a&inf_b | zero_b&inf_a)| 
		(inf_a | inf_b)&!int_times_inf;
	
	assign inexact = 
		(rounded & (!invalid)|
		overflow_tmp| 
		round_to_zero&overflow_tmp| 
		underflow&(!(zero_a|zero_b)))&!int_times_inf;

	assign underflow = 
		(format == `FP16) ? 
			(~ovf_ab&ovf_biased)| 
			(~|result_tmp[`FP16SW+`FP16EW-1:`FP16SW]) &
			!(ovf_ab&ovf_biased|ovf_postnorm) &
			!overflow&!invalid|(zero_a | zero_b) &
			!(nan_a|nan_b|inf_a|inf_b) :
		(format == `FP32) ? 
			(~ovf_ab&ovf_biased)| 
			(~|result_tmp[`FP32SW+`FP32EW-1:`FP32SW]) &
			!(ovf_ab&ovf_biased|ovf_postnorm) &
			!overflow&!invalid|(zero_a | zero_b) &
			!(nan_a|nan_b|inf_a|inf_b) :
		(format == `FP64) ? 
			(~ovf_ab&ovf_biased)| 
			(~|result_tmp[`FP64SW+`FP64EW-1:`FP64SW]) &
			!(ovf_ab&ovf_biased|ovf_postnorm) &
			!overflow&!invalid|(zero_a | zero_b) &
			!(nan_a|nan_b|inf_a|inf_b) : 0;
		
	// If overflow occurs and rounding mode equals round-to zero, 
	// result shall be rounded to largest representative number. 
	// e.x 0111101111111111.
	assign overflow_tmp = 
		(format == `FP16) ? 
			((ovf_ab&ovf_biased|ovf_postnorm&!underflow)|
			&result_tmp[`FP16SW+`FP16EW-1:`FP16SW]&!underflow)&!invalid :
		(format == `FP32) ? 
			((ovf_ab&ovf_biased|ovf_postnorm&!underflow)|
			&result_tmp[`FP32SW+`FP32EW-1:`FP32SW]&!underflow)&!invalid :
		(format == `FP64) ? 
			((ovf_ab&ovf_biased|ovf_postnorm&!underflow)|
			&result_tmp[`FP64SW+`FP64EW-1:`FP64SW]&!underflow)&!invalid : 
		0;
		
	assign overflow = overflow_tmp&!round_to_zero | int_times_inf;
		
	
	// Compute special results.
	assign product_nan = 
		(format == `FP16) ?
			{1'b0, {`FP16EW{1'b1}}, {(`FP16SW-1){1'b0}}, 1'b1} :
		(format == `FP32) ?
			{1'b0, {`FP32EW{1'b1}}, {(`FP32SW-1){1'b0}}, 1'b1}:
		(format == `FP64) ?
			{1'b0, {`FP64EW{1'b1}}, {(`FP64SW-1){1'b0}}, 1'b1} :
		0;

	assign product_zero = 
		(format == `FP16) ?
			{result_tmp[`FP16SW+`FP16EW], {(`FP16SW+`FP16EW){1'b0}}} :
		(format == `FP32) ?
			{result_tmp[`FP32SW+`FP32EW], {(`FP32SW+`FP32EW){1'b0}}} :
		(format == `FP64) ?
			{result_tmp[`FP64SW+`FP64EW], {(`FP64SW+`FP64EW){1'b0}}} : 
		0;
		
	assign product_overflow = 
		(format == `FP16) ?
			{result_tmp[`FP16SW+`FP16EW], 
			{`FP16EW{1'b1}}, {(`FP16SW){1'b0}}} :
		(format == `FP32) ?
			{result_tmp[`FP32SW+`FP32EW], 
			{`FP32EW{1'b1}}, {(`FP32SW){1'b0}}}:
		(format == `FP64) ?
			{result_tmp[`FP64SW+`FP64EW], 
			{`FP64EW{1'b1}}, {(`FP64SW){1'b0}}} :
		0;

	assign product_large = 
		(format == `FP16) ?
			{result_tmp[`FP16SW+`FP16EW], 
			{(`FP16EW-1){1'b1}}, 1'b0, {(`FP16SW){1'b1}}} :
		(format == `FP32) ?
			{result_tmp[`FP32SW+`FP32EW], 
			{(`FP32EW-1){1'b1}}, 1'b0, {(`FP32SW){1'b1}}}:
		(format == `FP64) ?
			{result_tmp[`FP64SW+`FP64EW], 
			{(`FP64EW-1){1'b1}}, 1'b0, {(`FP64SW){1'b1}}} :
		0;

	assign product_min = 
		(format == `FP16) ?
			{result_tmp[`FP16SW+`FP16EW], 
			{(`FP16EW-1){1'b0}}, 1'b1, {(`FP16SW){1'b0}}} :
		(format == `FP32) ?
			{result_tmp[`FP32SW+`FP32EW], 
			{(`FP32EW-1){1'b0}}, 1'b1, {(`FP32SW){1'b0}}}:
		(format == `FP64) ?
			{result_tmp[`FP64SW+`FP64EW], 
			{(`FP64EW-1){1'b0}}, 1'b1, {(`FP64SW){1'b0}}} :
		0;
	
	// Final product decided by exceptions.
	assign result = 
		invalid ? product_nan : 
		overflow ? product_overflow : 
		underflow ? product_zero : 
		round_to_zero & overflow_tmp & !int_times_inf ? product_large :
		result_tmp;
	
	assign exceps[0] = invalid;
	assign exceps[1] = inexact;
	assign exceps[2] = overflow;
	assign exceps[3] = underflow;
		
endmodule // rne64
