// ------------------------------------------------------------------
// File.......: rne_unit.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 12:19:50 CEST 2008
// Revision...: 1.0
// Description: Rounding, normalizing and exception unit.
// ------------------------------------------------------------------

`include "defines.v"

module rne_unit 
(
	fracs,	// Input from fraction bus.
	exps,		// Input form exponent bus.
	signs,	// Input from sign bus.
	format,	// Input from instrucion register.
	special, // Input form check special.
	mode,		// Input from mode register.
	exceps,	// Output exceptions.
	result	// Output. Rounded result.
);
   
 	// input(s)
	input [`FRACBUSOUT-1:0]		fracs;
	input [`EXPBUSOUT-1:0]		exps;
	input [`SIGNBUS/2-1:0]		signs;
	input [1:0]						format;
	input [1:0]						mode;
	input [15:0]					special;

	// output(s)
	output [`BUS/2-1:0]			result;
	output [7:0]					exceps;

	// wire(s)
	wire 								sign_rne_0;
	wire 								sign_rne_1;
	wire [2*`FP32SW+1:0]			frac_rne_0;
	wire [2*`FP64SW+1:0]			frac_rne_1;
	wire [`FP32EW+1:0]			exp_rne_0;
	wire [`FP64EW+1:0]			exp_rne_1;
	wire [`FP32W-1:0]				result_rne_0;
	wire [`FP64W-1:0]				result_rne_1;
	wire [7:0]						specials_rne_0;
	wire [7:0]						specials_rne_1;
	wire [3:0]						exceps_rne_0;
	wire [3:0]						exceps_rne_1;
	wire [1:0]						overflow;
	wire [1:0]						underflow;
	wire [1:0]						inexact;
	wire [1:0]						invalid;	
	// reg(s)
	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------

	rne32 #(`FP32SW, `FP32EW) rne_0
	(
		.frac			(frac_rne_0),
		.sign			(sign_rne_0),
		.exp			(exp_rne_0),
		.specials	(specials_rne_0),
		.mode			(mode),
		.format		(format),
		.result		(result_rne_0),
		.exceps		(exceps_rne_0)
	);
	
	rne64 #(`FP64SW, `FP64EW) rne_1
	(
		.frac			(frac_rne_1),
		.sign			(sign_rne_1),
		.exp			(exp_rne_1),
		.specials	(specials_rne_1),
		.mode			(mode),
		.format		(format),
		.result		(result_rne_1),
		.exceps		(exceps_rne_1)
	);
	

	
	// ---------------------------------------------------------------
	// Combinalional assign.
	// ---------------------------------------------------------------
	
	// Inputs to rounding logic.
	assign frac_rne_0 = fracs[2*(`FP32SW+1)-1:0];
	assign frac_rne_1 = fracs[`FRACBUSOUT-1:2*(`FP32SW+1)];
	
	// Two msb bits represents the overflow bits during exponent 
	// addition.
	assign exp_rne_0 = 
		(format == `FP16) ? 
			exps[`FP16EW+1:0] : 
		(format == `FP32) ? 
			exps[`FP32EW+1:0] : 
		0;
		
	assign exp_rne_1 = 
		(format == `FP16) ? 
			exps[`EXPBUSOUT-1:`FP16EW+2] : 
		(format == `FP32) ? 
			exps[`EXPBUSOUT-1:`FP32EW+2] :
		(format == `FP64) ? 
			exps[`FP64EW+1:0] : 
		0;
	
	assign sign_rne_0 = signs[0];
	
	assign sign_rne_1 = 
		(format == `FP64) ? 
			signs[0] :
		signs[1];
	
	assign specials_rne_0 = 
		{special[13:12], special[9:8], special[5:4], special[1:0]};

	assign specials_rne_1 = 
		(format == `FP64) ? 
			{special[13:12], special[9:8], special[5:4], special[1:0]} :
		{special[15:14], special[11:10], special[7:6], special[3:2]};
	
								
	assign underflow = 
		(format == `FP64) ? 
			{1'b0, exceps_rne_1[3]} :
		{exceps_rne_1[3], exceps_rne_0[3]};

	assign overflow = 
		(format == `FP64) ? 
			{1'b0, exceps_rne_1[2]} :
		{exceps_rne_1[2], exceps_rne_0[2]};

	assign inexact = 
		(format == `FP64) ? 
			{1'b0, exceps_rne_1[1]} :
		{exceps_rne_1[1], exceps_rne_0[1]};
		
	assign invalid = 
		(format == `FP64) ? 
			{1'b0, exceps_rne_1[0]} :
		{exceps_rne_1[0], exceps_rne_0[0]};
		
	assign exceps = {underflow, overflow, inexact, invalid};
						
	assign result = 
		(format == `FP16) ? 
			{result_rne_1[`FP16SW + `FP16EW:0],
			result_rne_0[`FP16SW + `FP16EW:0]} :
		(format == `FP32) ?
			{result_rne_1[`FP32SW + `FP32EW:0],
			result_rne_0[`FP32SW + `FP32EW:0]} :
		(format == `FP64) ? 
			result_rne_1[`FP64SW + `FP64EW:0] : 
		0;

endmodule // rne_unit
