// ------------------------------------------------------------------
// File.......: stage3.v
// Author.....: Espen Stenersen
// Date.......: Fri Apr 18 16:51:03 CEST 2008
// Revision...: 1.0
// Description: Stage three of pipeline.
// ------------------------------------------------------------------

`include "defines.v"

module stage3 
(
	DRF0,			// Input from fraction register.
	DRE0,			// Input from exponent register.
	DRS0,			// Input from sign register.
	format,		// Input from format register.
	mode,			// Input from rounding mode register.
	DRS1_out,	// Output to sign register.
	DRE1_out,	// Output to exponent register.
	DRF1_out,	// Output to fraction register.
	S0_out,		// Output to special values register.
	M2_out,		// Output to rounding mode register.
	IR2_out,		// Output to format register.
	clk,
	reset
);
   
 	// input(s)
	input [`FRACBUS-1:0]		DRF0;
	input [`EXPBUS-1:0]		DRE0;
	input [`SIGNBUS-1:0]		DRS0;
	input [1:0]					format;
	input [1:0]					mode;
	input							clk;
	input 						reset;

	// output(s)
	output [`FRACBUSOUT-1:0]DRF1_out;
	output [`SIGNBUS/2-1:0]	DRS1_out;
	output [`EXPBUSOUT-1:0]	DRE1_out; // + overflow/underflow bits.
	output [15:0]				S0_out;
	output [1:0]				M2_out;
	output [1:0]				IR2_out;

	// wire(s)
	wire [`FRACBUSOUT-1:0]	prods;
	wire [`SIGNBUS/2-1:0]	signs;
	wire [`EXPBUSOUT-1:0]	sums;
	wire [15:0]					specials;
	wire [3:0]					ints;
	wire [3:0]					infs;
	wire [3:0]					nans;
	wire [3:0]					zeroes;

	// reg(s)

	// ---------------------------------------------------------------
	// Module instantiations.
	// ---------------------------------------------------------------
	
	// Registers.
	ff #(`FRACBUSOUT) DRF1
	(
		.d			(prods),		// Data in.
		.q			(DRF1_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(`EXPBUSOUT) DRE1
	(
		.d			(sums),		// Data in.
		.q			(DRE1_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(`SIGNBUS/2) DRS1
	(
		.d			(signs),		// Data in.
		.q			(DRS1_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(16) S0
	(
		.d			(specials),	// Data in.
		.q			(S0_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(2) IR2
	(
		.d			(format),	// Data in.
		.q			(IR2_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(2) M2
	(
		.d			(mode),		// Data in.
		.q			(M2_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	// Computational units.
	chk_special #(`FRACBUS, `EXPBUS) chk_special 
	(
		.fracs	(DRF0),		// Input from significand bus.
		.exps		(DRE0),		// Input from exponent bus.
		.format	(format),	// Input.
		.infs		(infs),		// Output.
		.ints		(ints),		// Output.
		.nans		(nans),		// Output.
		.zeroes	(zeroes)		// Output.
	);
	
	mult_unit mult_unit
	(
		.fracs	(DRF0),		// Input from significand bus.
		.format	(format), 	// Input from instruction register.
		.prods	(prods)		// Output to significand bus.
	);
	
	exp_unit exp_unit
	(	
		.exps		(DRE0),		// Input from exponent bus.
		.format	(format),	// Input from instruction register.
		.sums		(sums)		// Output to exponent bus.
	);
	
	sign_unit sign_unit
	(
		.signs		(DRS0),	// Input signs from sign bus.
		.signs_comp	(signs)	// Output to sign bus.
	);
	
	assign specials = {ints, zeroes, infs, nans};
		
endmodule // stage3
