// ------------------------------------------------------------------
// File.......: sel_output_tb.v
// Author.....: Espen Stenersen
// Date.......: Thu Apr 24 21:39:26 CEST 2008
// Revision...: 1.0
// Description: For testing select output logic.
// ------------------------------------------------------------------

`include "defines.v"

module stage4 
(	
	start,		// Input.
	DRF1,			// Input from fraction register DRF1.
	DRE1,			// Input from exponent register DRE1.
	DRS1,			// Input from sign register DRS1.
	specials,	// Input form special values register S0.
	format,		// Input from format register IR2.
	mode,			// Input from rounding mode regiser M2.
	clear_excps,// Input. Clear exceptions.
	products,	// Output. Final result.
	exceptions,	// Output. Exceptions.
	ready,		// Output. Result ready.
	clk,			
	reset
);
   
	// Input(s)
	input								start;
	input [`FRACBUSOUT-1:0]		DRF1;
	input [`EXPBUSOUT-1:0]		DRE1;	//+ overflow/underflow bits.
	input [`SIGNBUS/2-1:0]		DRS1;
	input [15:0]					specials;
	input [1:0]						format;
	input [1:0]						mode;
	input [15:0]					clear_excps;
	input								clk;
	input								reset;
	
	// Output(s)
	output [`BUS-1:0]				products;
	output [15:0]					exceptions;
	output 							ready;
	
	// wire(s)
	wire [`BUS-1:0]				products;
	wire [15:0]						exceptions_tmp;
	wire								load_drh;
	wire								load_drlh;
	wire								load_drll;
	wire								load_excep_l;
	wire								load_excep_h;
	wire [15:0]						ex;
	wire [15:0]						clear;
	wire [7:0]						clear_l;
	wire [7:0]						clear_h;
	wire [7:0]						ex_h_in;
	wire [7:0]						ex_l_in;
	wire [7:0]						ex_h_out;
	wire [7:0]						ex_l_out;
	wire [`BUS-1:0]				prods;
	wire								ready_tmp;
	wire [`BUS/2-1:0]				result;
	wire [7:0]						exceps;
	wire [1:0]						format;

	// reg(s)
	
	
	// --------------------------------------------------------------
	// Module instantiation.
	// --------------------------------------------------------------
	
	// Product registers.
	reg_enable #(32) DRLL
	(
		.d					(prods[31:0]),			// Data in.
		.q					(products[31:0]),		// Data out.
		.enable			(load_drll),			// Enable bit.
		.clk  			(clk),		
		.reset			(reset)
	);
	
	reg_enable #(32) DRLH
	(
		.d					(prods[63:32]),		// Data in.
		.q					(products[63:32]),	// Data out.
		.enable			(load_drlh),			// Enable bit.
		.clk  			(clk),		
		.reset			(reset)
	);
	
	reg_enable #(64) DRH
	(
		.d					(prods[127:64]),		// Data in.
		.q					(products[127:64]),	// Data out.
		.enable			(load_drh),				// Enable bit.
		.clk  			(clk),		
		.reset			(reset)
	);

	// Exception registers.
	reg_enable #(8) EXCPL
	(
		.d					(ex_l_in),			// Data in.
		.q					(ex_l_out),			// Data out.
		.enable			(load_excep_l),	// Enable bit.
		.clk 				(clk),		
		.reset			(reset)
	);
	
	reg_enable #(8) EXCPH
	(
		.d					(ex_h_in),			// Data in.
		.q					(ex_h_out),			// Data out.
		.enable			(load_excep_h),	// Enable bit.
		.clk 				(clk),		
		.reset			(reset)
	);
	
	// Clear register. Written to in order to clear exceptions.
	// [unf p3 .. p0, ovf p3 .. p0, inx p3 .. p0, nan p3 .. p0]
	ff #(16) CLEAR
	(
		.d					(clear_excps),	// Data in.
		.q					(clear),			// Data out.
		.clk 				(clk),
		.reset			(reset)
	);
	
	// Ready register.
	reg_set #(1) READY
	(
		.set				(ready_tmp),
		.q					(ready),
		.clk 				(clk),
		.reset			(reset)
	);

	// Rounding unit.
	rne_unit rne_unit 
	(
		.fracs			(DRF1),		// Input from fraction bus.
		.exps				(DRE1),		// Input form exponent bus.
		.signs			(DRS1),		// Input from sign bus.
		.format			(format),	// Input from instrucion register.
		.special			(specials),	// Input form check special.
		.mode				(mode),		// Input from mode register.
		.exceps			(exceps),	// Output exceptions.
		.result			(result)		// Output. Rounded result.
	);
	
	// Output selector.
	sel_output sel_output
	(
		.result			(result),		// Input from rounding logic.
		.exceps			(exceps),		// Input from rounding logic.
		.format			(format),		// Input from format register.
		.start			(start),			// Input from start register.
		.products		(prods),			// Output to output register.
		.load_drh		(load_drh),		// Output to output register.
		.load_drlh		(load_drlh),	// Output to output register.
		.load_drll		(load_drll),	// Output to output register.
		.exceptions		(ex),				// Output to exception register.
		.load_excep_l	(load_excep_l),// Output to exception register.
		.load_excep_h	(load_excep_h),// Output to exception register.
		.reset			(reset),
		.clk 				(clk)
	);
	
	// ---------------------------------------------------------------
	// Assigns.
	// ---------------------------------------------------------------
	
		
	assign ready_tmp = 
		(format == `FP16) ? 
			load_drlh&start : 
		load_drh&start;
	
	assign clear_l = 
		{clear[13:12], clear[9:8], clear[5:4], clear[1:0]};

	assign clear_h = 
		{clear[15:14], clear[11:10], clear[7:6], clear[3:2]};
								
	assign ex_l_in = ex[7:0]&~clear_l;
	assign ex_h_in = ex[15:8]&~clear_h;
	
	assign exceptions = 
		(format == `FP64) ?
			{1'b0, 1'b0, ex_h_out[6], ex_l_out[6], 
			1'b0, 1'b0, ex_h_out[4], ex_l_out[4],
			1'b0, 1'b0, ex_h_out[2], ex_l_out[2],
			1'b0, 1'b0, ex_h_out[0], ex_l_out[0]} :						
		{ex_h_out[7:6], ex_l_out[7:6], 
		ex_h_out[5:4], ex_l_out[5:4],
		ex_h_out[3:2], ex_l_out[3:2], 
		ex_h_out[1:0], ex_l_out[1:0]};
endmodule // stage4
