// ------------------------------------------------------------------
// File.......: vec_fp_mult.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 10:30:15 CEST 2008
// Revision...: 1.0
// Description: Vectorized FP16/FP32/FP64 floating-point multiplier 
//              top module. Assembles the architecture.
// ------------------------------------------------------------------

`include "defines.v"

module vec_fp_mult 
(	
	start,				// Input. Starts computation.
	vectors,				// Input. FP vectors to be computed.
	format,				// Input. Format of vectors.
	mode,					// Input. Rounding mode.
	clear,				// Input. Clears specified exceptions.
	products,			// Output. Computed products.
	exceptions,			// Output. Exceptions raised.
	ready,				// Output. Output vector ready.
	clk,
	reset_n
);
   
 	// input(s)
	input								clk;
	input								reset_n;
	input								start;
	input [`BUS-1:0] 				vectors;
	input	[1:0]						format;
	input [1:0]						mode;
	input [15:0]					clear;

	// output(s)
	output [`BUS-1:0]				products;
	output [15:0]					exceptions;
	output 							ready;

	// wire(s)
	wire 								reset; 
	wire [`BUS/2-1:0]				DRH_to_stage2;
	wire [`BUS/2-1:0]				DRL_to_stage2;
	wire [1:0]						IR_to_stage2;
	wire [1:0]						IR_to_stage3;
	wire [1:0]						IR_to_stage4;
	wire [1:0]						M_to_stage2;
	wire [1:0]						M_to_stage3;
	wire [1:0]						M_to_stage4;
	wire [15:0]						S0_to_stage4;
	wire [`FRACBUS-1:0]			DRF_to_stage3;
	wire [`FRACBUSOUT-1:0]		DRF_to_stage4;
	wire [`EXPBUS-1:0]			DRE_to_stage3;
	wire [`EXPBUSOUT-1:0]		DRE_to_stage4;
	wire [`SIGNBUS-1:0]			DRS_to_stage3;
	wire [`SIGNBUS/2-1:0]		DRS_to_stage4;
	wire								start_to_stage1;
	wire								start_to_stage2;
	wire								start_to_stage3;
	wire								start_to_stage4;
	

	// ---------------------------------------------------------------
	// Module instantiation.	
	// ---------------------------------------------------------------
	
	// Registers. Keep track of start signal to set ready signal when 
	// needed.
	
	reg_enable #(1) ST0
	(
		.d			(start),					// Data in.
		.q 		(start_to_stage1),	// Data out.
		.enable	(start),					// Enable bit.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(1) ST1
	(
		.d			(start_to_stage1),	// Data in.
		.q 		(start_to_stage2),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(1) ST2
	(
		.d			(start_to_stage2),	// Data in.
		.q 		(start_to_stage3),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);

	ff #(1) ST3
	(
		.d			(start_to_stage3),	// Data in.
		.q 		(start_to_stage4),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	// Pipeline stage 1.
	stage1 stage1
	(
	 	.vectors			(vectors), 			// Input. Vectors.
		.start			(start),				// Input. Start computing.
		.format			(format),			// Input. Data format.
		.mode 			(mode),				// Input. Rounding mode.
		.DRH0_out 		(DRH_to_stage2),	// Output. [127:64] of input.
		.DRL0_out 	 	(DRL_to_stage2),	// Output. [63:0] of input.
		.IR0_out 		(IR_to_stage2),	// Output. Format.
		.M0_out 			(M_to_stage2),		// Output. Rounding mode.
		.clk 				(clk),
		.reset			(reset)
	);
	
	// Pipeline stage 2.
	stage2 stage2 
	(
		.DRH0				(DRH_to_stage2),	// Input from register DRH0.
		.DRL0 			(DRL_to_stage2),	// Input from register DRL0.
		.format			(IR_to_stage2),	// Input from register IR0.
		.mode 			(M_to_stage2),   	// Input from mode register M0.
		.DRF0_out 		(DRF_to_stage3),	// Output to multipliers.
		.DRE0_out 		(DRE_to_stage3),	// Output to exponent adders.
		.DRS0_out 		(DRS_to_stage3),	// Output to sign computation.
		.IR1_out 		(IR_to_stage3),	// Output.
		.M1_out 			(M_to_stage3),		// Output.
		.clk 				(clk),
		.reset			(reset)
	);
	
	// Pipeline stage 3.
	stage3 stage3 
	(
		.DRF0 			(DRF_to_stage3),	// Input from register DRF0.
		.DRE0 			(DRE_to_stage3),	// Input from register DRE0.
		.DRS0 			(DRS_to_stage3),	// Input from register DRS0.
		.format 			(IR_to_stage3),	// Input from format register.
		.mode 			(M_to_stage3),		// Input from  mode register.
		.DRS1_out 		(DRS_to_stage4),	// Output to sign register.
		.DRE1_out 		(DRE_to_stage4),	// Output to exponent register.
		.DRF1_out 		(DRF_to_stage4),	// Output to fraction register.
		.S0_out 			(S0_to_stage4),	// Output to special register.
		.M2_out 			(M_to_stage4),		// Output to mode register.
		.IR2_out 		(IR_to_stage4),	// Output to format register.
		.clk 				(clk),
		.reset			(reset)
	);
	
	// Pipeline stage 4 & 5.
	stage4 stage4 
	(	
		.start			(start_to_stage4),// Input.
		.DRF1				(DRF_to_stage4),	// Input from register DRF1.
		.DRE1				(DRE_to_stage4),	// Input from register DRE1.
		.DRS1				(DRS_to_stage4),	// Input from register DRS1.
		.specials		(S0_to_stage4),	// Input form register S0.
		.format			(IR_to_stage4),	// Input from register IR2.
		.mode				(M_to_stage4),		// Input from regiser M2.
		.clear_excps 	(clear),				// Input. Clear exceptions.
		.products		(products),			// Output. Final result.
		.exceptions		(exceptions),		// Output. Exceptions.
		.ready			(ready),				// Output. Result ready.
		.clk 				(clk),			
		.reset			(reset)
	);
	
	
	// Internal active high reset.
	assign reset = !reset_n;
	
endmodule // vec_fp_mult
