# ------------------------------------------------------------------
# Author: 		Espen Stenersen.
# Date:			Spring 2008.
# Description: Automatic synthesis script. Synthesizes the given
#              architecture for a given target technology and 
#              different clock frequencies and switching activity
#              profiles.
# ------------------------------------------------------------------

# ------------------------------------------------------------------
# Set script variables.
# ------------------------------------------------------------------

set HOME /home/espens/master

set 90NM 0
set 90NM 0

set date [clock format [clock seconds] -format {%m_%d_%Y_%M_%H}]

# ------------------------------------------------------------------
# Choose architecture to synthesize.
# ------------------------------------------------------------------

puts "\nArchitectures.\n" 
puts "\[1\]: Architecture One."
puts "\[2\]: Architecture Two."
puts -nonewline "\nChoose architecture: " 
flush stdout

while {1} {
	gets stdin key
	
	if {$key == "1"} {
		set DESIGN design1
		break
	} elseif {$key == "2"} {
		set DESIGN design2
		break
	} else {
		puts -nonewline "\nChoose architecture: "
	}
}

# Source timer script.
source $HOME/$DESIGN/scripts/timer.tcl

# ------------------------------------------------------------------
# Choose target library.
# ------------------------------------------------------------------

puts "\nAvailable target libraries.\n" 
puts "\[1\]: 65nm nomainal vaules"
puts "\[2\]: 90nm nomainal vaules"
puts -nonewline "\nChoose target library: " 
flush stdout

while {1} {
	gets stdin key
	
	if {$key == "1"} {
		set lib $HOME/$DESIGN/scripts/setlib65_nom.tcl
		set 65NM 1
		break
	} elseif {$key == "2"} {
		set lib $HOME/$DESIGN/scripts/setlib90_nom.tcl
		set 90NM 1
		break
	} else {
		puts -nonewline "\nChoose target library: " 
		set 65NM 0
		set 90NM 0
	}
}

source $lib

# Choose switching activity profile.
puts "\nAvailable switching-activity profiles\n"
puts "\[1\] Typical input-data"
puts "\[2\] 100 \% FP16 input-data"
puts "\[3\] 100 \% FP32 input-data"
puts "\[4\] 100 \% FP64 input-data"

while {1} {
	puts -nonewline "\nChoose switching-activity profile: " 
	flush stdout
	gets stdin key
	
	if {$key == "1"} {
		set saif typical
		break
	} elseif {$key == "2"} {
		set saif fp16
		break
	} 	elseif {$key == "3"} {
		set saif fp32
		break
	} 	elseif {$key == "4"} {
		set saif fp64
		break
	}	else {
		puts -nonewline "\nChoose switching-activity profile: "
	}
}

# Start execution timer.
start_timer

set mhz 400

puts "Generates design with clock frequencies 400 MHz, 300 MHz and 200 MHz.\n"

for {set i 0} {$i < 3} {incr i} {
	remove_design -all
	
	# Read design files, analyze and elaborate.
	source $HOME/$DESIGN/scripts/analyze_design.tcl 
	
	# Set top module.	
	current_design vec_fp_mult
	
	# Link design.
	link
	
	# Create clock.
	if {$mhz == 400} {set period 2.5}
	if {$mhz == 300} {set period 3.3}
	if {$mhz == 200} {set period 5.0}

	
	create_clock -period $period [get_ports clk]
	
	# Set constraints.	
	set_max_area 0
	set_max_total_power 0
	
	# Compile.	
	if {$DESIGN == "design1"} {
		set saif_file "$HOME/$DESIGN/synthesis/bwd_toggle1_$mhz\_$saif.saif"
	} elseif {$DESIGN == "design2"} {
		set saif_file "$HOME/$DESIGN/synthesis/bwd_toggle2_$mhz\_$saif.saif"
	} else {
		puts "ERROR: Could not find SAIF file.\n"
		puts "DESIGN: $DESIGN\n"
		break
	}
	read_saif -input $saif_file -instance vec_fp_mult_stimuli_tb/DUT
	compile_ultra
	read_saif -input $saif_file -instance vec_fp_mult_stimuli_tb/DUT
	
	
	# Create reports.
	if {$90NM == "1"} {
		set area_file "$HOME/$DESIGN/docs/design/90nm/$mhz\_$saif\_area_report.txt"
		set power_file "$HOME/$DESIGN/docs/design/90nm/$mhz\_$saif\_power_report.txt"
		set timing_file "$HOME/$DESIGN/docs/design/90nm/$mhz\_$saif\_timing_report.txt"
		set design_file "$HOME/$DESIGN/synthesis/90nm/$mhz\_$saif\_vec_fp_mult.db"
	} elseif {$65NM == "1"} {
		set area_file "$HOME/$DESIGN/docs/design/65nm/$mhz\_$saif\_area_report.txt"
		set power_file "$HOME/$DESIGN/docs/design/65nm/$mhz\_$saif\_power_report.txt"
		set timing_file "$HOME/$DESIGN/docs/design/65nm/$mhz\_$saif\_timing_report.txt"
		set design_file "$HOME/$DESIGN/synthesis/65nm/$mhz\_$saif\_vec_fp_mult.db"
	} else {
		puts "ERROR: Report paths not defined!\n"
		break
	}

	# ---------------------------------------------------------------
	# Save results in database format.
	# ---------------------------------------------------------------

	write -xg_force_db -f db -h -o $design_file

	# ---------------------------------------------------------------
	# Generate reports.
	# ---------------------------------------------------------------

	puts "\n******************************************************************************\n"
	puts "\nGenerating design reports..."

	report_area -hierarchy > $area_file
	report_power -analysis_effort high -hier > $power_file
	report_timing > $timing_file

	puts "\nArea report was written to:\n\t $area_file"
	puts "\nPower report was written to:\n\t $power_file"
	puts "\nTiming report was written to:\n\t $timing_file"

	puts "\n\n\nDesign was written to:\n\t $design_file"

	puts "\n******************************************************************************\n"

			
	set mhz [expr $mhz-100]
}

# Stop execution timer.
stop_timer