// ------------------------------------------------------------------
// File.......: debug.v
// Author.....: Espen Stenersen
// Date.......: Sat Apr 26 01:04:00 CEST 2008
// Revision...: 1.0
// Description: Tasks for debuging design. Reports errors and signal 
//              status at different verbosity level.
// ------------------------------------------------------------------

	reg v1;
	reg v2;
	reg v3;
	reg v4;

	// ---------------------------------------------------------------
	// Verbosity levels:
	// 0:	Only final report.
	// 1:	Signal events and updates.
	// 2: Error messages.
	// 3: Elaborated error messages with product vectors, exception
	//    vectors and input vectors that caused the error.
	// 4:	1 and 3 combined.
	// ---------------------------------------------------------------

	task verbosity;
		input [2:0] verbosity;
		begin
			case (verbosity)
				0: begin
					v1 = 0; v2 = 0; v3 = 0; v4 = 0;
				end
				// Signal updates.
				1: begin
					v1 = 1;
					print_header;
				end
				
				// Error messages.
				2: begin
					v2 = 1;
				end
				
				// Elaborated messages.
				3: begin
					v3 = 1;
				end
				
				// Elaborated messages with signal updates.
				4: begin
					v4 = 1;
				end
				
				default: begin
					v1 = 0; v2 = 0; v3 = 0; v4 = 0;
				end
			endcase
		end
	endtask

	// ---------------------------------------------------------------
	// Prints error elaborated error messages. 
	// ---------------------------------------------------------------
	
	always @ (ready) begin
		if ((v2|v3)&(ready == 1)) begin
			if ((products != dw_products)|
				(exceptions != dw_exceptions)) begin
				print_error;
			end
		end
	end

	
	// ---------------------------------------------------------------
	// Updates the signal status print-out.
	// ---------------------------------------------------------------
	always @ (reset_n) begin
		if (v1|v4) 
			$display("@ %0d\t\t| reset_n\t\t| %b", ($time)/2, reset_n);
	end
	always @ (start) begin
		if (v1|v4) 
			$display("@ %0d\t\t| start\t\t\t| %b", ($time)/2, start);
	end
	always @ (ready) begin
		if (v1) 
			$display("@ %0d\t\t| ready\t\t\t| %b", ($time)/2, ready);
		if (v4&ready) print_error;
		if (v4&!ready) 
			$display("@ %0d\t\t| ready\t\t\t| %b", ($time)/2, ready);
	end
	always @ (clear) begin
		if (v1|v4) 
			$display("@ %0d\t\t| clear\t\t\t| %b %b %b %b", ($time)/2, 
						clear[15:12], clear[11:8], clear[7:4], clear[3:0]);
	end
	always @ (ready) begin
		if (ready == 1) begin 
			if ((v1)&(products != dw_products)) 
				$display("@ %0d\t\t| products\t\t| ERROR!", ($time)/2);
		end
	end
	always @ (ready) begin
		if (ready == 1) begin
			if ((v1)&(exceptions != dw_exceptions))
				$display("@ %0d\t\t| exceptions\t\t| ERROR!", ($time)/2);
		end
	end
	always @ (format) begin
		if (v1|v4) begin
			case (format)
				`FP16: begin
					$display("@ %0d\t\t| format\t\t| 16-bit floating-point (\'b%b)", 
					($time)/2, format);
				end
				`FP32: begin
					$display("@ %0d\t\t| format\t\t| 32-bit floating-point (\'b%b)", 
					($time)/2, format);
				end
				`FP64: begin
					$display("@ %0d\t\t| format\t\t| 64-bit floating-point (\'b%b)",
					($time)/2, format);
				end
			endcase
		end
	end
	always @ (mode) begin
		if (v1|v4) begin
			case (mode)
				`EVEN: begin
					$display("@ %0d\t\t| mode\t\t\t| Round-to-nearest even (\'b%b)", 
					($time)/2, mode);
				end
				`PINF: begin
					$display("@ %0d\t\t| mode\t\t\t| Round-to-positive infinity (\'b%b)", 
					($time)/2, mode);
				end
				`NINF: begin
					$display("@ %0d\t\t| mode\t\t\t| Round-to-negative infinity (\'b%b)", 
					($time)/2, mode);
				end
				`ZERO: begin
					$display("@ %0d\t\t| mode\t\t\t| Round-to zero (\'b%b)", 
					($time)/2, mode);
				end
			endcase
		end
	end
	always @ (exceptions) begin
		if (v4) begin
			$display("@ %0d\t\t| exceptions\t\t| %b %b %b %b", $time/2,
			exceptions[15:12], exceptions[11:8], exceptions[7:4], exceptions[3:0]);
		end
	end


	// ---------------------------------------------------------------
	// Prints error messages.
	// ---------------------------------------------------------------
	
	task print_error;
		begin
			if (i_failed == 1) print_header;
			
			if ((products != dw_products)) begin
				$write("-----------------------------------------");
				$write("-----------------------------------------\n");
				$display("@ %0d\t\t| products\t\t| ERROR: Products faild.", 
				$time/2);
				$write("-----------------------------------------");
				$write("-----------------------------------------\n");
			end
			else if ((exceptions != dw_exceptions)) begin
				$write("-----------------------------------------");
				$write("-----------------------------------------\n");
				$display("@ %0d\t\t| exceptions\t\t| ERROR: Exceptions failed.", 
				$time/2);
				$write("-----------------------------------------");
				$write("-----------------------------------------\n");
			end
			if (v4) begin
				$display("@ %0d\t\t| ready\t\t\t| %b", $time/2, ready);
				$write("-----------------------------------------");
				$write("-----------------------------------------\n");
			end
			if ((v3|v4)) begin
				if (format == `FP64) begin
					$display("A0\t\t| %b", testmem[4*i_total + 0]);
					$display("B0\t\t| %b", testmem[4*i_total + 1]);
					$display("A1\t\t| %b", testmem[4*i_total + 2]);
					$display("B1\t\t| %b", testmem[4*i_total + 3]);
					$write("-----------------------------------------");
					$write("-----------------------------------------\n");
				end
				else begin
					$display("A0\t\t| %b", testmem[8*i_total + 0]);
					$display("B0\t\t| %b", testmem[8*i_total + 1]);
					$display("A1\t\t| %b", testmem[8*i_total + 2]);
					$display("B1\t\t| %b", testmem[8*i_total + 3]);
					$display("C0\t\t| %b", testmem[8*i_total + 4]);
					$display("D0\t\t| %b", testmem[8*i_total + 5]);
					$display("C1\t\t| %b", testmem[8*i_total + 6]);
					$display("D1\t\t| %b", testmem[8*i_total + 7]);
					$write("-----------------------------------------");
					$write("-----------------------------------------\n");
				end	
			end
		
			$display("DUT [127:64]\t| %b", products[127:64]);
			$display("DUT [ 63:0 ]\t| %b", products[63:0]);
			$write("-----------------------------------------");
			$write("-----------------------------------------\n");
			$display("DW  [127:64]\t| %b", dw_products[127:64]);
			$display("DW  [ 63:0 ]\t| %b", dw_products[63:0]);
			$write("-----------------------------------------");
			$write("-----------------------------------------\n");
			$display("DUT [15:0]\t| %b %b %b %b (underflow overflow inexact invalid)", 
			exceptions[15:12], exceptions[11:8], exceptions[7:4], exceptions[3:0]);
			$write("-----------------------------------------");
			$write("-----------------------------------------\n");
			$display("DW  [15:0]\t| %b %b %b %b (underflow overflow inexact invalid)", 
			dw_exceptions[15:12], dw_exceptions[11:8], dw_exceptions[7:4], dw_exceptions[3:0]);
			$write("-----------------------------------------");
			$write("-----------------------------------------\n");
		end
	endtask
	// ---------------------------------------------------------------
	// Prints final report.
	// --------------------------------------------------------------- 
	
	task print_report;
		begin
			$display("\n\n");
			$write("****************************************");
			$write("****************************************\n");
			$display("\nFINAL REPORT\n");
			print_format(format);
			print_mode(mode);
			$display("Input statistics");
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			$display("Total invalid inputs\t\t: %0d", i_nan);
			$display("Total zero inputs\t\t: %0d", i_zero);
			$display("Total infinity inputs\t\t: %0d", i_inf);
			$display("Total infintiy times zero\t: %0d", i_inf_times_zero);
			$display("Total invalid times any number\t: %0d", i_nan_times_any);
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			$display("Total input vectors\t\t: %0d", VECTORS);
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			$display("");
			$display("Ouput statistics");
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			$display("Total overflowed products\t: %0d", i_ovf);
			$display("Total underflowed products\t: %0d", i_unf);
			$display("Total invalid products\t\t: %0d", i_inv);
			$display("Total inexact products\t\t: %0d", i_inx);
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			if (format == `FP64) begin
				// Times two because each product vector consists of 
				// two products
				$display("Total products\t\t\t: %0d", 2*i_total);
				//$display("Total products passed\t\t: %0d", 2*i_passed);
				//$display("Total products failed\t\t: %0d", 2*i_failed);
				//$write("----------------------------------------");
				//$write("----------------------------------------\n");
				$display("Total product vectors\t\t: %0d", i_total);
				$display("Total products vectors passed\t: %0d", i_passed);
				$display("Total products vectors failed\t: %0d", i_failed);
			end
			else begin
				// Times four because each product vector consists of 
				// four products
				$display("Total products\t\t\t: %0d", 4*i_total);
				//$display("Total products passed\t\t: %0d", 4*i_passed);
				//$display("Total products failed\t\t: %0d", 4*i_failed);
				//$write("----------------------------------------");
				//$write("----------------------------------------\n");
				$display("Total product vectors\t\t: %0d", i_total);
				$display("Total products vectors passed\t: %0d", i_passed);
				$display("Total products vectors failed\t: %0d", i_failed);
			end
			$write("----------------------------------------");
			$write("----------------------------------------\n");
			$display("\n\n");
			if (i_failed > 0) begin
				$display("Test finished without success!");
			end
			else begin
				$display("Test finished successfully!");
			end
			$display("");
			$write("****************************************");
			$write("****************************************\n");
		end
	endtask
	
	
	task print_header;
		begin
			$write("=========================================");
			$write("=========================================\n");
			$display("Time (cycle)\t| Signal\t\t| Event");
			$write("=========================================");
			$write("=========================================\n");
		end
	endtask
	
	
	// ---------------------------------------------------------------
	// Prints rounding mode.
	// ---------------------------------------------------------------
	task print_mode;
		input [1:0]	mode;
		begin
			case (mode)
				`EVEN: begin
					$display("Rounding mode\t\t\t: Round-to-nearest even\n");
				end
				`PINF: begin
					$display("Rounding mode\t\t\t: Round-to-positive infinity\n");
				end
				`NINF: begin
					$display("Rounding mode\t\t\t: Round-to-negative infinity\n");
				end
				`ZERO: begin
					$display("Rounding mode\t\t\t: Round-to zero\n");
				end
			endcase
		end
	endtask

	// ---------------------------------------------------------------
	// Prints data format tested.
	// ---------------------------------------------------------------
	task print_format;
		input [5:0] data_format;
		begin
			if (data_format == `FP16) begin
				$display("Data format\t\t\t: 16-bit floating-point (FP16)");
			end
			else if (data_format == `FP32) begin
				$display("Data format\t\t\t: 32-bit floating-point (FP32)");
			end
			else if (data_format == `FP64) begin
				$display("Data format\t\t\t: 64-bit floating-point (FP64)");
			end
		end
	endtask