// ------------------------------------------------------------------
// File.......: vec_fp_mult_stimuli_tb.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 29 14:19:15 CEST 2008
// Revision...: 1.0
// Description: Generates switching activity information to the  
//					 synopsys power analasys tools.
// ------------------------------------------------------------------


`include "defines.v"

`timescale 	1ps/1ps
`define 		CLK_PERIOD 5000


module vec_fp_mult_stimuli_tb;

	parameter 			FP16STEP = 4;
	parameter 			FP32STEP = 4;
	parameter 			FP64STEP = 2;
	parameter			FP16VECTORS = 100;
	parameter			FP32VECTORS = 0;
	parameter			FP64VECTORS = 0;
	
	// wire(s)
	wire [127:0]		products;
	wire [15:0]			exceptions;
	wire					ready;

	// reg(s)
	reg 					start;
	reg [127:0]			vectors;
	reg [1:0]			format;
	reg [1:0]			mode;
	reg [15:0]			clear;
	reg 					clk;
	reg 					reset_n;
	
	reg [`FP16W-1:0]	fp16testmem [0:FP16VECTORS];
	reg [`FP32W-1:0]	fp32testmem [0:FP32VECTORS];
	reg [`FP64W-1:0]	fp64testmem [0:FP64VECTORS];
	
	integer 				i_vec;
	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------
	vec_fp_mult DUT
	(	
		.start		(start),			// Input. Starts computation.
		.vectors		(vectors),		// Input. FP vectors to be computed.
		.format		(format),		// Input. Format of vectors.
		.mode			(mode),			// Input. Rounding mode.
		.clear		(clear),			// Input. Clears exceptions.
		.products	(products),		// Output. Computed products.
		.exceptions	(exceptions),	// Output. Exceptions raised.
		.ready		(ready),			// Output. Output vector ready.
		.clk 			(clk),
		.reset_n		(reset_n)
	);

	
	initial begin
		clk = 1; reset_n = 0; start = 0; vectors = 0; clear = 0;
		format = 0; mode = `EVEN;
		`include "d1_tracefile.v"
		$dumpfile("toggle1_200_fp16.vcd");
		$readmemb("fp16testcases.txt", fp16testmem);
		$readmemb("fp32testcases.txt", fp32testmem);
		$readmemb("fp64testcases.txt", fp64testmem);
		
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) reset_n = 1;
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		
		// Round-to-nearest even.
		for (i_vec = 0; i_vec < FP16VECTORS; i_vec = i_vec + FP16STEP) begin
			@ (posedge clk) begin
				format = `FP16;
				start = 1;
				vectors[1*`FP16W-1:0*`FP16W] <= fp16testmem[i_vec + 0];
				vectors[2*`FP16W-1:1*`FP16W] <= fp16testmem[i_vec + 1];
				vectors[3*`FP16W-1:2*`FP16W] <= fp16testmem[i_vec + 2];
				vectors[4*`FP16W-1:3*`FP16W] <= fp16testmem[i_vec + 3];
			end
		end
		for (i_vec = 0; i_vec < FP32VECTORS; i_vec = i_vec + FP32STEP) begin
			@ (posedge clk) begin
				format = `FP32;
				start = 1;
				vectors[1*`FP32W-1:0*`FP32W] <= fp32testmem[i_vec + 0];
				vectors[2*`FP32W-1:1*`FP32W] <= fp32testmem[i_vec + 1];
				vectors[3*`FP32W-1:2*`FP32W] <= fp32testmem[i_vec + 2];
				vectors[4*`FP32W-1:3*`FP32W] <= fp32testmem[i_vec + 3];
			end
		end
		for (i_vec = 0; i_vec < FP64VECTORS; i_vec = i_vec + FP64STEP) begin
			@ (posedge clk) begin
				format = `FP64;
				start = 1;
				vectors[1*`FP64W-1:0*`FP64W] <= fp64testmem[i_vec + 0];
				vectors[2*`FP64W-1:1*`FP64W] <= fp64testmem[i_vec + 1];
			end
		end
		// Round-to-positive infinity.
		mode = `PINF;
		for (i_vec = FP16VECTORS; i_vec < 2*FP16VECTORS; i_vec = i_vec + FP16STEP) begin
			@ (posedge clk) begin
				format = `FP16;
				start = 1;
				vectors[1*`FP16W-1:0*`FP16W] <= fp16testmem[i_vec + 0];
				vectors[2*`FP16W-1:1*`FP16W] <= fp16testmem[i_vec + 1];
				vectors[3*`FP16W-1:2*`FP16W] <= fp16testmem[i_vec + 2];
				vectors[4*`FP16W-1:3*`FP16W] <= fp16testmem[i_vec + 3];
			end
		end
		for (i_vec = FP32VECTORS; i_vec < 2*FP32VECTORS; i_vec = i_vec + FP32STEP) begin
			@ (posedge clk) begin
				format = `FP32;
				start = 1;
				vectors[1*`FP32W-1:0*`FP32W] <= fp32testmem[i_vec + 0];
				vectors[2*`FP32W-1:1*`FP32W] <= fp32testmem[i_vec + 1];
				vectors[3*`FP32W-1:2*`FP32W] <= fp32testmem[i_vec + 2];
				vectors[4*`FP32W-1:3*`FP32W] <= fp32testmem[i_vec + 3];
			end
		end
		for (i_vec = FP64VECTORS; i_vec < 2*FP64VECTORS; i_vec = i_vec + FP64STEP) begin
			@ (posedge clk) begin
				format = `FP64;
				start = 1;
				vectors[1*`FP64W-1:0*`FP64W] <= fp64testmem[i_vec + 0];
				vectors[2*`FP64W-1:1*`FP64W] <= fp64testmem[i_vec + 1];
			end
		end
		// Round-to-negative inifity.
		mode = `NINF;
		for (i_vec = 2*FP16VECTORS; i_vec < 3*FP16VECTORS; i_vec = i_vec + FP16STEP) begin
			@ (posedge clk) begin
				format = `FP16;
				start = 1;
				vectors[1*`FP16W-1:0*`FP16W] <= fp16testmem[i_vec + 0];
				vectors[2*`FP16W-1:1*`FP16W] <= fp16testmem[i_vec + 1];
				vectors[3*`FP16W-1:2*`FP16W] <= fp16testmem[i_vec + 2];
				vectors[4*`FP16W-1:3*`FP16W] <= fp16testmem[i_vec + 3];
			end
		end
		for (i_vec = 2*FP32VECTORS; i_vec < 3*FP32VECTORS; i_vec = i_vec + FP32STEP) begin
			@ (posedge clk) begin
				format = `FP32;
				start = 1;
				vectors[1*`FP32W-1:0*`FP32W] <= fp32testmem[i_vec + 0];
				vectors[2*`FP32W-1:1*`FP32W] <= fp32testmem[i_vec + 1];
				vectors[3*`FP32W-1:2*`FP32W] <= fp32testmem[i_vec + 2];
				vectors[4*`FP32W-1:3*`FP32W] <= fp32testmem[i_vec + 3];
			end
		end
		for (i_vec = 2*FP64VECTORS; i_vec < 3*FP64VECTORS; i_vec = i_vec + FP64STEP) begin
			@ (posedge clk) begin
				format = `FP64;
				start = 1;
				vectors[1*`FP64W-1:0*`FP64W] <= fp64testmem[i_vec + 0];
				vectors[2*`FP64W-1:1*`FP64W] <= fp64testmem[i_vec + 1];
			end
		end
		// Round-to zero.
		mode = `ZERO;
		for (i_vec = 3*FP16VECTORS; i_vec < 4*FP16VECTORS; i_vec = i_vec + FP16STEP) begin
			@ (posedge clk) begin
				format = `FP16;
				start = 1;
				vectors[1*`FP16W-1:0*`FP16W] <= fp16testmem[i_vec + 0];
				vectors[2*`FP16W-1:1*`FP16W] <= fp16testmem[i_vec + 1];
				vectors[3*`FP16W-1:2*`FP16W] <= fp16testmem[i_vec + 2];
				vectors[4*`FP16W-1:3*`FP16W] <= fp16testmem[i_vec + 3];
			end
		end
		for (i_vec = 3*FP32VECTORS; i_vec < 4*FP32VECTORS; i_vec = i_vec + FP32STEP) begin
			@ (posedge clk) begin
				format = `FP32;
				start = 1;
				vectors[1*`FP32W-1:0*`FP32W] <= fp32testmem[i_vec + 0];
				vectors[2*`FP32W-1:1*`FP32W] <= fp32testmem[i_vec + 1];
				vectors[3*`FP32W-1:2*`FP32W] <= fp32testmem[i_vec + 2];
				vectors[4*`FP32W-1:3*`FP32W] <= fp32testmem[i_vec + 3];
			end
		end
		for (i_vec = 3*FP64VECTORS; i_vec < 4*FP64VECTORS; i_vec = i_vec + FP64STEP) begin
			@ (posedge clk) begin
				format = `FP64;
				start = 1;
				vectors[1*`FP64W-1:0*`FP64W] <= fp64testmem[i_vec + 0];
				vectors[2*`FP64W-1:1*`FP64W] <= fp64testmem[i_vec + 1];
			end
		end
		// Empty pipeline.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		start = 0;
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		$finish;
	end
	
	// Toggles clearing of exceptions.
	always @ (ready) begin
		if (ready == 1) begin
			clear = 16'b1111111111111111;
		end
		else begin
			clear = 16'b0000000000000000;
		end
	end
	
	
	// clock generator.
	always #(`CLK_PERIOD/2) clk = !clk;
	
endmodule // vec_fp_mult_stimuli_tb
