// ------------------------------------------------------------------
// File.......: vec_fp_mult_tb.v
// Author.....: Espen Stenersen
// Date.......: Thu Apr 17 13:49:28 CEST 2008
// Revision...: 1.0
// Description: Testbench for top module vec_fp_mult.
// ------------------------------------------------------------------

`include "../rtl/defines.v"

//`timescale 
`define CLK_PERIOD 	1

module vec_fp_mult_tb;

	`include "../tb/defines_tb.v"
	`include "../tb/debug.v"
	
	parameter W 			= `FP16W;
	parameter SW 			= `FP16SW;
	parameter EW 			= `FP16EW;
	parameter FORMAT		= `FP16;
	parameter MODE			= `ZERO;
	parameter VECTORS		= 100000;

	// wire(s)
	wire [`BUS-1:0]		products;
	wire [15:0]				exceptions;
	wire 						ready;
	wire						exceptions_failed;
	wire						products_failed;
	wire [3:0]				nan, inf, zero;
	
	wire [`BUS-1:0]		dw_products;
	wire [15:0]				dw_exceptions;
	wire [2*`BUS-1:0]		dw_vectors;
	
	// reg(s)
	reg [W-1:0]				testmem [0:VECTORS-1];
	reg [`BUS-1:0]			vectors;
	reg [W-1:0]				A0, B0, A1, B1;
	reg [1:0]				format;
	reg [1:0]				mode;
	reg [15:0]				clear;
	reg 						start;
	reg						clk;
	reg						reset_n;

	
	reg [2:0]				dw_mode;
	
	// Counters.
	integer 					i_vec, i_ans, i_passed, i_failed, i_total;
	integer 					i_nan, i_zero, i_inf, i_inf_times_zero;
	integer					step, i_ovf, i_unf, i_inv, i_inx;
	integer					i_nan_times_any;

	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------
	vec_fp_mult DUT
	(	
		.start				(start),			// Input. Starts computation.
		.vectors				(vectors),		// Input. FP vectors.
		.format				(format),		// Input. Format of vectors.
		.mode					(mode),			// Input. Rounding mode.
		.clear				(clear),			// Input. Clears exceptions.
		.products			(products),		// Output. Computed products.
		.exceptions			(exceptions),	// Output. Exceptions raised.
		.ready				(ready),			// Output. Output ready.
		.clk 					(clk),
		.reset_n				(reset_n)
	);
	
	dw_vec_fp_mult dw_vec_fp_mult 
	(
		.dw_vectors			(dw_vectors),		// Input from testbench.
		.dw_mode				(dw_mode),			// Input from testbench.
		.format				(format),			// Input from testbench.
		.dw_products		(dw_products),		// Output to testbench.
		.dw_exceptions		(dw_exceptions)	// Output to testbench.
	);

		
	// ---------------------------------------------------------------
	// Initials.
	// ---------------------------------------------------------------
	
	// Generate stimuli.
	initial begin	
		
		// ------------------------------------------------------------
		// Verbosity levels:
		// 	0:	Only final report.
		// 	1:	Signal events and updates.
		// 	2: Error messages.
		// 	3: Elaborated error messages with product vectors, 
		//       exception vectors and input vectors that caused the 
		//       error.
		// 	4:	1 and 3 combined.
		// ------------------------------------------------------------
	
		verbosity(3);
	
		initialize;	// Call to initialize task.
		
		
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) reset_n = 1;
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		
		for (i_vec = 0; i_vec < VECTORS; i_vec = i_vec + step) begin
			@ (posedge clk) begin
				start = 1; 
		
				if (format == `FP16) begin
					vectors[1*`FP16W-1:0*`FP16W] <= testmem[i_vec + 0];
					vectors[2*`FP16W-1:1*`FP16W] <= testmem[i_vec + 1];
					vectors[3*`FP16W-1:2*`FP16W] <= testmem[i_vec + 2];
					vectors[4*`FP16W-1:3*`FP16W] <= testmem[i_vec + 3];
					A0 <= testmem[i_vec + 0];
					B0 <= testmem[i_vec + 1];
					A1 <= testmem[i_vec + 2];
					B1 <= testmem[i_vec + 3];
				end
				else if (format == `FP32) begin
					vectors[1*`FP32W-1:0*`FP32W] <= testmem[i_vec + 0];
					vectors[2*`FP32W-1:1*`FP32W] <= testmem[i_vec + 1];
					vectors[3*`FP32W-1:2*`FP32W] <= testmem[i_vec + 2];
					vectors[4*`FP32W-1:3*`FP32W] <= testmem[i_vec + 3];
					A0 <= testmem[i_vec + 0];
					B0 <= testmem[i_vec + 1];
					A1 <= testmem[i_vec + 2];
					B1 <= testmem[i_vec + 3];
				end
				else if (format == `FP64) begin
					vectors[1*`FP64W-1:0*`FP64W] <= testmem[i_vec + 0];
					vectors[2*`FP64W-1:1*`FP64W] <= testmem[i_vec + 1];
					A0 <= testmem[i_vec + 0];
					B0 <= testmem[i_vec + 1];
				end
				else begin
					vectors <= 0;
					A0 <= 0;
					B0 <= 0;
					A1 <= 0;
					B1 <= 0;
				end
			end
		end
		// Empty pipeline.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		start = 0;
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		@ (posedge clk) // wait cycle.
		print_report;
		$finish;
	end
	
	// ---------------------------------------------------------------
	// Sequential test logic.
	// ---------------------------------------------------------------

	// clock generator.
	always #`CLK_PERIOD clk = !clk;
	
	
	
	// Monitor / checker.
	always @ (ready) begin
		if (reset_n == 0) begin
			i_ans <= 0;
			i_total <= 0;
		end
	
		// When products and exceptions are ready at output.
		if (ready == 1) begin
			i_total <= i_total + 1;

			if (format == `FP64) begin
				i_ans <= i_ans + 4;
			end
			else begin
				i_ans <= i_ans + 8;
			end
			
			
			if ((products != dw_products)|
				(exceptions != dw_exceptions)) begin
				i_failed = i_failed + 1;
			end
			else begin
				i_passed = i_passed + 1;
			end
		end
	end

	
	// ---------------------------------------------------------------
	// Combinaional test logic.
	// ---------------------------------------------------------------
	
	// Clears exceptions when arised.
/*	always @ (ready) begin
		if (ready) begin
			clear = 'hffff;
		end
		else if (!ready) begin
			clear = 'h0000;
		end
	end*/
	
	
	// ---------------------------------------------------------------
	// Produces test statistics.
	// ---------------------------------------------------------------
	// Counts infity inputs.
	integer i0;
	always @ (inf or start) begin
		for (i0 = 0; i0 < 4; i0 = i0 + 1) begin
			if (inf[i0]&start) i_inf = i_inf + 1;
		end
	end
	// Counts zero inputs.
	integer i1;
	always @ (zero or start) begin
		for (i1 = 0; i1 < 4; i1 = i1 + 1) begin
			if (zero[i1]&start) i_zero = i_zero + 1;
		end
	end
	// Counts invalid inputs.
	integer i2;
	always @ (nan or start) begin
		for (i2 = 0; i2 < 4; i2 = i2 + 1) begin
			if (nan[i2]&start) i_nan = i_nan + 1;
		end
	end
	// Counts infinity times zero inputs.
	integer i3;
	always @ (inf or zero or start) begin
		for (i3 = 0; i3 < 2; i3 = i3 + 2) begin
			if (inf[i3]&zero[i3+1]&start) 
				i_inf_times_zero = i_inf_times_zero + 1;
		end
		for (i3 = 0; i3 < 2; i3 = i3 + 2) begin
			if (inf[i3+1]&zero[i3]&start) 
				i_inf_times_zero = i_inf_times_zero + 1;
		end
	end
	// Counts invalid times any number (not invalid times invalid).
	integer i4;
	always @ (nan or start) begin
		for (i4 = 0; i4 < 2; i4 = i4 + 1) begin
			if (nan[i4]&!nan[i4+1]&start) 
				i_nan_times_any = i_nan_times_any + 1;
		end
	end
	// Counts underflows.
	integer o0;
	always @ (ready or exceptions) begin
		for (o0 = 0; o0 < 4; o0 = o0 + 1) begin
			if (exceptions[12 + o0]&ready) i_unf = i_unf + 1;
		end
	end
	// Counts overflows.
	integer o1;
	always @ (ready or exceptions) begin
		for (o1 = 0; o1 < 4; o1 = o1 + 1) begin
			if (exceptions[8 + o1]&ready) i_ovf = i_ovf + 1;
		end
	end
	// Counts inexacts.
	integer o2;
	always @ (ready or exceptions) begin
		for (o2 = 0; o2 < 4; o2 = o2 + 1) begin
			if (exceptions[4 + o2]&ready) i_inx = i_inx + 1;
		end
	end
	// Counts invalids.
	integer o3;
	always @ (ready or exceptions) begin
		for (o3 = 0; o3 < 4; o3 = o3 + 1) begin
			if (exceptions[0 + o3]&ready) i_inv = i_inv + 1;
		end
	end	
	
	
	// ---------------------------------------------------------------
	// Assigns.
	// ---------------------------------------------------------------
		
	assign dw_vectors = 
		(format == `FP16) ?
		{testmem[i_ans + 7], testmem[i_ans + 6],
		testmem[i_ans + 5], testmem[i_ans + 4],
		testmem[i_ans + 3], testmem[i_ans + 2],
		testmem[i_ans + 1], testmem[i_ans + 0]} :
		(format == `FP32) ?
		{testmem[i_ans + 7], testmem[i_ans + 6],
		testmem[i_ans + 5], testmem[i_ans + 4],
		testmem[i_ans + 3], testmem[i_ans + 2],
		testmem[i_ans + 1], testmem[i_ans + 0]} :
		(format == `FP64) ?
		{testmem[i_ans + 3], testmem[i_ans + 2],
		testmem[i_ans + 1], testmem[i_ans + 0]} : 0;

	
	assign nan[0] = (&A0[W-2:SW])&(|A0[SW-1:0]);
	assign inf[0] = (&A0[W-2:SW])&(~|A0[SW-1:0]);
	assign zero[0] = (~|A0[W-2:SW])&(~|A0[SW-1:0]);
	
	assign nan[1] = (&A1[W-2:SW])&(|A1[SW-1:0]);
	assign inf[1] = (&A1[W-2:SW])&(~|A1[SW-1:0]);
	assign zero[1] = (~|A1[W-2:SW])&(~|A1[SW-1:0]);
	
	assign nan[2] = (&B0[W-2:SW])&(|B0[SW-1:0]);
	assign inf[2] = (&B0[W-2:SW])&(~|B0[SW-1:0]);
	assign zero[2] = (~|B0[W-2:SW])&(~|B0[SW-1:0]);
	
	assign nan[3] = (&B1[W-2:SW])&(|B1[SW-1:0]);
	assign inf[3] = (&B1[W-2:SW])&(~|B1[SW-1:0]);
	assign zero[3] = (~|B1[W-2:SW])&(~|B1[SW-1:0]);
	
	
	
	// ---------------------------------------------------------------
	// Tasks.
	// ---------------------------------------------------------------
	
	
	task initialize;
		begin
			set_mode(MODE); format = FORMAT;
			
			clk = 1; reset_n = 0; clear = 0; start = 0;
			A0 = 0; B0 = 0; B1 = 0; A1 = 0; vectors = 0;
			i_vec = 0; i_ans = 0; i_passed = 0; i_failed = 0;
			i_nan = 0; i_zero = 0; i_inf = 0; i_inf_times_zero = 0;
			i_nan_times_any = 0; i_ovf = 0; i_unf = 0; i_inv = 0; 
			i_inx = 0;
			
			// Opens correct testcase readfile.
			if (format == `FP16) begin
				step = 4;
				$readmemb(`FP16TESTCASES, testmem);
			end
			else if (format == `FP32) begin
				step = 4;
				$readmemb(`FP32TESTCASES, testmem);
			end
			else if (format == `FP64) begin
				step = 2;
				$readmemb(`FP64TESTCASES, testmem);
			end
			$display("");
		end
	endtask

	
	// Sets rounding mode for both floating-point multipliers.
	task set_mode;
		input [1:0]	r_mode;
		begin
			case (r_mode)
				`EVEN: begin
					mode = `EVEN;
					dw_mode = `DW_EVEN;
				end
				`PINF: begin
					mode = `PINF;
					dw_mode = `DW_PINF;
				end
				`NINF: begin
					mode = `NINF;
					dw_mode = `DW_NINF;
				end
				`ZERO: begin
					mode = `ZERO;
					dw_mode = `DW_ZERO;
				end
				default: begin 
					$display("Not valid rounding mode!");
					$finish;
				end
			endcase
		end
	endtask
	
endmodule // vec_fp_mult_tb
