// ------------------------------------------------------------------
// Author:			Espen Stenersen.
// Date:				Spring 2008.
// Description:	Generates FP16, FP32 and FP64 testvectors including
//                "random" special inputs such as infinity x zero, 
//   					nans ... Testfiles are written to fp16testfiles.txt,
// 					fp32testfiles.txt and fp64testfiles.txt.
// ------------------------------------------------------------------

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <xlocale.h>
#include <stdlib.h>
#include <unistd.h>

#define FP16 0
#define FP32 1
#define FP64 2
#define FP16WIDTH 16;
#define FP32WIDTH 32;
#define FP64WIDTH 64;
#define FP16EXPONENT 5;
#define FP32EXPONENT 8;
#define FP64EXPONENT 11;

void usage();
void generate(int format, int testcases);
void generate_nan(int width, int exponent);
void generate_zero(int width, int exponent);
void generate_infinity(int width, int exponent);
void generate_special(int width, int exponent);
void generate_random(int width, int exponent);

FILE *f;

int main (int argc, char const *argv[])
{
	int format;
	
	// Initializes the random generator.
	srand(time(0) * getpid()); 
	
	if (argc < 3) usage();
	else
	{
		if ( strcmp("-fp16", argv[1]) == 0 ) format = FP16;
		else if ( strcmp("-fp32", argv[1]) == 0 ) format = FP32;
		else if ( strcmp("-fp64", argv[1]) == 0 ) format = FP64;
		else usage();
		
		generate(format, atoi(argv[2]));
	}
	return 0;
}

void generate(int format, int testcases)
{
	int i = 0; 
	int width = 0; 
	int exponent = 0;
	int random = 0;
	
	switch( format )
	{
		case FP16: width = FP16WIDTH; exponent = FP16EXPONENT; f = fopen ("fp16testcases.txt", "wt"); break;
		case FP32: width = FP32WIDTH; exponent = FP32EXPONENT; f = fopen ("fp32testcases.txt", "wt"); break;
		case FP64: width = FP64WIDTH; exponent = FP64EXPONENT; f = fopen ("fp64testcases.txt", "wt"); break;
	}
		
	// Genrates nan x nan.
	generate_nan(width, exponent);
	generate_nan(width, exponent);

	// Generates zero x infinity.
	generate_infinity(width, exponent);
	generate_infinity(width, exponent);

	// Generates zero x zero.
	generate_zero(width, exponent);
	generate_zero(width, exponent);

	// Generates zero z infinity.
	generate_zero(width, exponent);
	generate_infinity(width, exponent);

	// Generates infinity x nan.
	generate_infinity(width, exponent);
	generate_nan(width, exponent);

	// Generates zero x nan.
	generate_zero(width, exponent);
	generate_nan(width, exponent);
	
	i = 12;
	while (i < testcases)
	{
		random = rand() % 999;
		if (random == 14)
		{
			generate_special(width, exponent);
		}	
		else
		{
			generate_random(width, exponent);
		}
		i++;
	}
	fclose (f);
}

void generate_random(int width, int exponent)
{
	int j = 0;
	int normalized = 0;
	int bit = 0;
	
	while (j < width) 
	{
		bit = rand() % 2;
		if (j < 1) fprintf(f, "%d", bit);
		else if (j < exponent)
		{
			if (bit == 1) normalized++;
			fprintf(f, "%d", bit);
		}
		else
		{
			if (j == exponent)
			{
				if (normalized < 1)
					fprintf(f, "%d", 1);
				else
					fprintf(f, "%d", bit);
			}
			else
				fprintf(f, "%d", bit);
		}
		j++;
	}
	fprintf(f, "\n");
}
// Generate random special input vectos. e.x zero x infinity.
void generate_special(int width, int exponent)
{
	int random = rand() % 6;
	
	// Genrates nan x nan.
	if (random == 0)
	{
		generate_nan(width, exponent);
		generate_nan(width, exponent);
	}
	// Generates zero x infinity.
	else if (random == 1)
	{
		generate_infinity(width, exponent);
		generate_infinity(width, exponent);
	}
	// Generates zero x zero.
	else if (random == 2)
	{
		generate_zero(width, exponent);
		generate_zero(width, exponent);
	}
	// Generates zero z infinity.
	else if (random == 3)
	{
		generate_zero(width, exponent);
		generate_infinity(width, exponent);
	}
	// Generates infinity x nan.
	else if (random == 4)
	{
		generate_infinity(width, exponent);
		generate_nan(width, exponent);
	}
	// Generates zero x nan.
	else if (random == 5)
	{
		generate_zero(width, exponent);
		generate_nan(width, exponent);
	}
}

// Generate NaN vectors.
void generate_nan(int width, int exponent)
{
	int i = 0;
	int bit = rand() % 2;
	while (i < width)
	{
		if (i < 1) fprintf(f, "%d", bit);
		else if (i < exponent + 1) fprintf(f, "%d", 1);
		else if (i < width - 1) fprintf(f, "%d", 0);
		else fprintf(f, "%d", 1);
		i++;
	}
	fprintf(f, "\n");
}

// Generate zero vectors.
void generate_zero(int width, int exponent)
{
	int i = 0;
	int bit = rand() % 2;
	while (i < width)
	{
		if (i < 1) fprintf(f, "%d", bit);
		else fprintf(f, "%d", 0);
		i++;
	}
	fprintf(f, "\n");
}

// Generate infinity vectors.
void generate_infinity(int width, int exponent)
{
	int i = 0;
	int bit = rand() % 2;
	while (i < width)
	{
		if (i < 1) fprintf(f, "%d", bit);
		else if (i < exponent + 1) fprintf(f, "%d", 1);
		else fprintf(f, "%d", 0);
		i++;
	}
	fprintf(f, "\n");
}

// Prints user info.
void usage()
{
	printf("\nGenerates normalized IEEE conforming testvectors of desired format.\n");
	printf("\nUsage: generatetestvectors <format> <number of testcases>\n");
	printf("      -fp16: 16-bit floating-point vectors.\n");
	printf("      -fp32: 32-bit floating-point vectors.\n");
	printf("      -fp64: 64-bit floating-point vectors.\n");
	printf("\n   Ex: generatetestvectors -fp16 10000\n\n");
	printf("\n");
}